package CoreBroker;

import java.io.PrintWriter;
import java.util.Vector;
import java.util.HashMap;
import java.io.File;

import WSAMI.WSAMIAbstractDocument;
import WSAMI.WSAMICustomizerDocument;
import WSAMI.WSAMIServiceDocument;

import java.io.IOException;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletResponse;


import CoreBroker.DeployedService;
import CoreBroker.BrokerWrapper; 
import CoreBroker.BrokerFactory;

////////////////////////////////
import org.w3c.dom.*; 
import javax.xml.parsers.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
//////////////////////////

import CoreBroker.Utils;

public class AdminServlet extends HttpServlet {

    private String baseURL;
    private String baseURLServices;
    private String ndAddress;
    private String contextRealPath;
    private String context;
    
    private String wsamiConfigXML = null;
    private String wsamiInstalledServicesXML = null;
    
    private final String DEFAULT_PORT = "8080";
    private final String DEFAULT_CONTEXT = "wsami";
    private final String DEFAULT_SERVICESDIR = "services";    
    private final String ND_INITIAL_DEPLOY_FILE = "NDDeploy.xml";
    private final String DEFAULT_CONFIGFILE = "wsami.xml";
    private final String DEFAULT_INSTALLEDSERVICESFILE = "wsamiInstalledServices.xml";    
    private final String DEPLOY_NAMESPACE = "http://www-rocq.inria.fr/arles/wsami";

    private String fileSeparator;
    
    private final int HTTP_ERRORCODE = 423;


    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */    
    public void destroy() {
        // unregister from slp: making a call in this method is not a good
        // solution. the nd.stop is never called
        // the right solution is with servicelifecycle destroy
        try {
            // NDService.NDServiceXLocator loc = new NDService.NDServiceXLocator();
            // NDService.NDPortType nd = loc.getND1(new java.net.URL("http://localhost:8080/wsami/services/ND"));
            // nd.stop(); 
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        
        
        
        super.destroy();
        Syslog.debug(this,"AdminServlet.destroy()");
    }
    

    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */    
    private void setBaseURL() {
        String port = null;
        String hostaddr = null;

        /*
         * retrieve initialization parameters 
         * values are set in web.xml file
         */
        port = getInitParameter("port");
        if (port == null)
            port = DEFAULT_PORT;
        
        context = getInitParameter("context");
        if (context == null)
            context = DEFAULT_CONTEXT;

        /*
         *  retrieve host address. default is 127.0.0.1
         */
        try{
            hostaddr = java.net.InetAddress.getLocalHost().getHostAddress(); 
        } catch ( java.net.UnknownHostException ex) {
            hostaddr = null;
        }
        if (hostaddr == null)
            hostaddr = "127.0.0.1";

        baseURL =  "http://"+hostaddr+":"+port+"/"+context;        
    }
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    private void setBaseURLServices() {
        String services = null;
        
        /*
         * retrieve initialization parameters 
         * values are set in web.xml file
         
        services = getInitParameter("services");
        if (services == null)
            services = DEFAULT_SERVICESDIR;
        
        baseURLServices =  baseURL+"/"+services;
         */

        baseURLServices = "http://localhost:8080/services";
        
    }
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    private void setContextRealPath() {
        contextRealPath = getServletConfig().getServletContext().getRealPath("/");
        if (!contextRealPath.endsWith(fileSeparator))
            contextRealPath = contextRealPath.concat(fileSeparator);
    }

    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    public void init() throws ServletException {
        
        java.util.Properties props = System.getProperties();
        fileSeparator = props.getProperty("file.separator");                       

        boolean ErrorInNDdeployment = false;

        try {
           /*
            * retrieve Broker Factory Name from initialization parameters 
            * value is set in web.xml file
            */            
            String factoryBrokerWrapperName = getInitParameter("factoryname");
            if (factoryBrokerWrapperName == null)
                throw new CBException("init00","No Factory Name found in AdminServlet parameters"); 
            //try {
                BrokerFactory.setFactory(factoryBrokerWrapperName);        
                /*
            } catch (CBException ex) {
                throw new CBException("init00", "Error initializing the Core Broker: "
                    +ex.getDescription());
            }
                 */
                
            Syslog.debug(this,"AdminServlet.init()");

            /*
             *  set baseURL, baseURLServices and contextRealPath
             */
            setBaseURL();
            setBaseURLServices();
            setContextRealPath();
            
            /*
             *  try to retrieve from jar files implementing the
             *  Core Broker the XML files containing the data
             *  for ND Deployment
             */
            ClassLoader c = this.getClass().getClassLoader();
            java.io.InputStream ndXML = c.getResourceAsStream(ND_INITIAL_DEPLOY_FILE);
            Document NDdeployDoc = InStr2Document(ndXML,true);

            /*
             *  deploy ND service. parameter is
             *  false because ND must not be registred on itself
             */
            deployServices(NDdeployDoc, null, false);      

            /*
             *  If ND is well deployed on Core Broker I can obtain the
             *  endpoint where it has been deployed
             */
            ndAddress = CoreBroker.BrokerWrapper.getLocalNDAddr();

            
            /* 
             *  to startup SLP registration and lookup
             */            
            try {     
                BrokerWrapper broker = BrokerFactory.getServerInstance();
                
                String wsamiBinDir = getInitParameter("wsamiBinDir");
                if (wsamiBinDir == null)
                    wsamiBinDir = "/opt/wsami/dist/pda/bin";
                broker.setWSAMIBinDir(wsamiBinDir);                

                String URAddr = getInitParameter("UR");
                broker.setURAddr(URAddr);                

                NDService.NDPortType nd = broker.getNDStub(ndAddress);
                
                nd.initialization();
                
            } catch (java.rmi.RemoteException ex) {
                throw new CBException("deploy02","The address of the ND service is not correct"); 
            }

        } catch( CBException ex ) {
            Syslog.fatal(this,"Error in Naming & Discovery Service initialization:"+
                                "\n - "+ex.getDescription());
            ErrorInNDdeployment = true;
            
        }            

        /*
         *  if there are no errors in ND deployment
         *  try to deploy other services from configuration file            
         */
        if (!ErrorInNDdeployment) {            
            wsamiConfigXML = getInitParameter("config");
            if (wsamiConfigXML == null)
                wsamiConfigXML = DEFAULT_CONFIGFILE;        

            wsamiConfigXML = contextRealPath.concat(wsamiConfigXML);
            wsamiInstalledServicesXML = contextRealPath.concat(DEFAULT_INSTALLEDSERVICESFILE);

            try {
                java.io.FileInputStream fis = new java.io.FileInputStream(wsamiConfigXML);
                Document configDeployDoc = InStr2Document(fis, true);

                deployServices(configDeployDoc, null, true);  
                readFromInstalledServicesXML();

		Syslog.writelog("System started");
                
            } catch (java.io.FileNotFoundException ex) {
                Syslog.warning(this,wsamiConfigXML+" file not found: no services deployed.");
            } catch (CBException ex) {
                Syslog.warning(this,"Error reading configuration file: "+"\n - "+ex.getDescription());
            }
            

            
        }
        
    }
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    private void readFromInstalledServicesXML() {                
        DocumentBuilderFactory dbf = null;        
        DocumentBuilder db = null;
        Document installedDoc = null;
        
        try {
            dbf = DocumentBuilderFactory.newInstance();        
            db = dbf.newDocumentBuilder();
        } catch (javax.xml.parsers.ParserConfigurationException ex) {
            Syslog.error(this,"Error reading install file ");
        }
         
        try {
            installedDoc = db.parse(new java.io.FileInputStream(wsamiInstalledServicesXML));
        } catch (org.xml.sax.SAXException ex) {
            Syslog.warning(this,wsamiInstalledServicesXML+" does not exist.");
        } catch (java.io.FileNotFoundException ex) {
            Syslog.warning(this,wsamiInstalledServicesXML+" does not exist.");            
        } catch (java.io.IOException e) {
            Syslog.error(this,"Error reading file "+wsamiInstalledServicesXML);
        }

        NodeList installNodes = installedDoc.getElementsByTagName("install");

        for (int k=0; k<installNodes.getLength(); k++) {

            Node installNode = installNodes.item(k);
            String serviceName = installNode.getAttributes().getNamedItem("name").getNodeValue();                

            Vector filesVector = new Vector();
            NodeList fileList = installNode.getChildNodes();

            for (int i=0; i<fileList.getLength(); i++) {
                if (fileList.item(i).getNodeName().equals("file")) {
                    NamedNodeMap at = fileList.item(i).getAttributes();
                    String name = at.getNamedItem("name").getNodeValue();
                    filesVector.add(name);
                }
            }
            DeployedService ds = CoreBroker.BrokerWrapper.getDeployedService(serviceName);                
            if (ds != null) {
                String[] files = new String[filesVector.size()];
                filesVector.copyInto(files);
                ds.setFiles(files); 
            }
        }
    }
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    private void addServiceToInstalledServicesFile(String serviceName, String[] installedFiles) {
        DocumentBuilderFactory dbf = null;
        DocumentBuilder db = null;
        Document installedServicesDoc = null;
        
        try {
            dbf = DocumentBuilderFactory.newInstance();        
            db = dbf.newDocumentBuilder();
        } catch (javax.xml.parsers.ParserConfigurationException ex) {
            Syslog.error(this,"Error creating install/deploy file ");
        }        
        
        boolean wsamiInstalledServicesXMLExists = true;
        try {
            installedServicesDoc = db.parse(new java.io.FileInputStream(wsamiInstalledServicesXML));
        } catch (org.xml.sax.SAXException ex) {
            Syslog.warning(this,"Error "+wsamiInstalledServicesXML+" does not exist.");
        } catch (java.io.FileNotFoundException ex) {
            Syslog.warning(this,wsamiInstalledServicesXML+" does not exist.");
            wsamiInstalledServicesXMLExists = false;
        } catch (java.io.IOException e) {
            Syslog.warning(this,"Error reading file "+wsamiInstalledServicesXML);
            wsamiInstalledServicesXMLExists = false;
        }

        if (!wsamiInstalledServicesXMLExists) {
            installedServicesDoc = db.newDocument();
            Element root = installedServicesDoc.createElement("installation");

            root.setAttribute("xmlns","http://www-rocq.inria.fr/arles/wsami");
            root.setAttribute("xmlns:xsi","http://www.w3.org/2001/XMLSchema-instance");
            root.setAttribute("xsi:schemaLocation","http://www-rocq.inria.fr/arles/wsami deploy.xsd");
            installedServicesDoc.appendChild(root);   
        }    

        /*
         *  verify if the service is already defined in installed services file
         *  and if it is, remove the node from the document
         *  before adding the new service node definition
         */
        NodeList checkingServiceNodes = installedServicesDoc.getElementsByTagName("install");
        for (int k=0; k<checkingServiceNodes.getLength(); k++) {
            Node checkingServiceNode = checkingServiceNodes.item(k);
            String nameServiceNode = checkingServiceNode.getAttributes().getNamedItem("name").getNodeValue();
            if (nameServiceNode.equals(serviceName)) {
                Node parentServiceNode = checkingServiceNode.getParentNode();
                parentServiceNode.removeChild(checkingServiceNode);
            }
        }          
        
        /*
         *  add the new service installation definition to
         *  the document
         */
        Element root = installedServicesDoc.getDocumentElement();
        Element installNode = installedServicesDoc.createElement("install");
        installNode.setAttribute("name",serviceName);
        root.appendChild(installNode);

        for (int k=0; k<installedFiles.length; k++) {
            Element fileNode = installedServicesDoc.createElement("file");
            fileNode.setAttribute("name",installedFiles[k]);
            installNode.appendChild(fileNode);
        }

        /*
         *  write the new service installation document to file
         */
        try {            
            java.io.FileOutputStream fos = new java.io.FileOutputStream(wsamiInstalledServicesXML);
            CoreBroker.Utils.printDocToStream(installedServicesDoc, fos);            
            if (!wsamiInstalledServicesXMLExists)
		Syslog.info(this,wsamiInstalledServicesXML+" created.");
        } catch (java.io.FileNotFoundException ex) {
            Syslog.warning(this,"not found 2");
        }        
    }
    
    /**
    * deploys a list of services and installs them if required as in installNodes
     *      information. Updates wsami.xml and wsamiInstalledServices.xml files.
    * @param doc the XML document containg the services (1 or more) to deploy. 
     *          the format must be compatible with deploy.xsd
    * @param installNodes null if the services in doc have already been installed
     *          in the server. otherwise it has been constructed with the
     *          format of deploy_install.xsd
    * @param registerND true if the services, after the deployment on core broker,
     *          must be registred on ND. true for all services except for ND.
    * @return
    */
    private void deployServices(Document doc, HashMap installNodes, boolean registerND) 
            throws CBException {
                

        BrokerWrapper broker = BrokerFactory.getServerInstance();
        NodeList serviceList = doc.getElementsByTagNameNS(DEPLOY_NAMESPACE,"service");
        
        boolean installationRequired = (installNodes != null);
        if (serviceList == null || serviceList.getLength() == 0) {
            throw new CBException("deploy00","No Service description in deployment document"); 
        }
        
        for (int i=0; i<serviceList.getLength(); i++)
        {
            boolean deployed = false;      
            
            String serviceName = serviceList.item(i).getAttributes().getNamedItem("name").getNodeValue();
            String wau = BrokerWrapper.getTagValue(serviceList.item(i), "wau");

            if (!CoreBroker.BrokerWrapper.isServiceDeployed(serviceName)) {
                try {
                    String[] installedFiles = null;
                    
                    if (installationRequired) {
                        Node installNode = (Node)installNodes.get(serviceName);
                        installedFiles = installServices(installNode);                    
                    }

                    broker.verify(serviceList.item(i), contextRealPath, context, baseURLServices);
                    broker.deployService(serviceList.item(i), contextRealPath, context);

                    if (installationRequired) {
                    /*
                     *  updates CoreBroker DeployedService data
                     *  about installed files
                     *  
                     */
                        DeployedService ds = CoreBroker.BrokerWrapper.getDeployedService(serviceName);
                        ds.setFiles(installedFiles);                    
                        addServiceToInstalledServicesFile(serviceName, installedFiles);
                    /*
                     *  update wsami.xml configuration file
                     *  with all the new deployed services 
                     */
                        addServiceToConfigFile(serviceName, serviceList.item(i));
                    }

                    
                    deployed = true;
                    
                    Syslog.info(this,"Service "+ serviceName +" successfully deployed. ");
                    
                } catch (CBException ex) {
                    Syslog.error(this,"Error deploying service "+ serviceName
                        +"\n - Error verifying XML deploy file: "
                        +"\n   - "+ex.getDescription());
                }
        
           
                /*
                 *  if registerND is true try to register the 
                 *  service reference on NS service:
                 *      wau: is directly obtained from the deploy document
                 *      wsu and wcu: are obtained from the names on the local 
                 *                  filesystem of the documents wsd and wcd 
                 */
                if (deployed && registerND) {                
                    String wsu = BrokerWrapper.getTagValue(serviceList.item(i), "wsd"); 
                    if (wsu != null)
                        wsu = baseURL+"/".concat(wsu);

                    String wcu = BrokerWrapper.getTagValue(serviceList.item(i), "wcd");                
                    if (wcu != null)
                        wcu = baseURL+"/".concat(wcu);

                    try {
                        NDService.NDPortType nd = broker.getNDStub(ndAddress);
                        nd.register(serviceName, wau, wsu, wcu);
                    } catch (java.rmi.RemoteException ex) {

                        broker.undeployService(serviceName/*serviceList.item(i)*/);                    
                        Syslog.error(this,"Error in service "+ wau + "registration: "
                                    +"\n - " +ex.getMessage());   
                    }                     
                }
            } else 
                Syslog.warning(this,"Error deploying service "+ serviceName
                        +": service already deployed.");

        }
    }
    
    /**
    * undeploys a list of services and removes them. 
     * Updates wsami.xml and wsamiInstalledServices.xml files.
    * @param doc the list of service to be undeployed (deploy.xsd)
    */
    private void undeployServices(Document doc) throws CBException{
        
        BrokerWrapper broker = BrokerFactory.getServerInstance();
        NodeList serviceList = doc.getElementsByTagNameNS(DEPLOY_NAMESPACE,"service");

        if (serviceList == null || serviceList.getLength() == 0) {
            throw new CBException("deploy05","No Service description in undeployment document"); 
        }

        for (int i=0; i<serviceList.getLength(); i++)
        {
            String serviceName = serviceList.item(i).getAttributes().getNamedItem("name").getNodeValue();

            if (CoreBroker.BrokerWrapper.isServiceDeployed(serviceName)) {
                removeInstalledFiles(serviceName);
                broker.undeployService(serviceName/*serviceList.item(i)*/  /*, contextRealPath, context*/);
                removeServiceFromConfigFile(serviceName);
                removeServiceFromInstalledServicesFile(serviceName);

                Syslog.info(this,"Service "+ serviceName +" successfully undeployed. ");
                
                /*
                 * unregister service from ND service repository
                 */
                String wau = BrokerWrapper.getTagValue(serviceList.item(i), "wau");
                if (wau == null)
                    throw new CBException("deploy06","wau not defined for Service");

                try {
                    NDService.NDPortType nd = broker.getNDStub(ndAddress);
                    nd.unRegister(serviceName);
                } catch (java.rmi.RemoteException ex) {
                    throw new CBException("deploy02","Error in service unregistration: "+ex.getMessage()); 
                }
            } else 
                Syslog.warning(this,"Error undeploying service "+ serviceName
                        +": service not deployed.");
        }
    }
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    private void removeInstalledFiles(String serviceName) {

        DeployedService ds = CoreBroker.BrokerWrapper.getDeployedService(serviceName);
        
        if (ds != null) {
            String[] filesToRemove = ds.getFiles();                
            for (int i=0; i<filesToRemove.length; i++) {

                File file=new File(filesToRemove[i]);
                file.delete();
                String[] filesInThisDir = file.getParentFile().list();
                if (filesInThisDir == null || filesInThisDir.length==0)
                    file.getParentFile().delete();
            }
        }
    }
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    private String[] installServices(Node doc) {
        Vector listOfInstalledFiles = new Vector();
        
        try {   
            String servicesImplPath = null;
            ServletContext contextServices = getServletConfig().getServletContext().getContext("/services");
            if (contextServices != null) {
                servicesImplPath = contextServices.getRealPath("/");
                if (!servicesImplPath.endsWith(fileSeparator))
                    servicesImplPath = servicesImplPath.concat(fileSeparator);
            }

            String servicesImplPathClassFiles = servicesImplPath+"WEB-INF"+fileSeparator+"classes"+fileSeparator;
            String servicesImplPathJarFiles = servicesImplPath+"WEB-INF"+fileSeparator+"lib"+fileSeparator;

            ServletContext context = getServletConfig().getServletContext();
            String rootPath = context.getRealPath("/");
            if (!rootPath.endsWith(fileSeparator))
                rootPath = rootPath.concat(fileSeparator);
         
            NamedNodeMap at1 = doc.getAttributes();
            for (int j=0; j<at1.getLength(); j++) {
                    Node n = at1.item(j);
            }
            
            String serviceName = at1.getNamedItem("name").getNodeValue();
            
            
            NodeList serviceList = doc.getChildNodes();
            for (int i=0; i<serviceList.getLength(); i++)
            {
              if (serviceList.item(i).getNodeName().equals("file")) {
                NamedNodeMap at = serviceList.item(i).getAttributes();
                for (int j=0; j<at.getLength(); j++) {
                    Node n = at.item(j);
                }
                String type = at.getNamedItem("type").getNodeValue();
                String name = at.getNamedItem("name").getNodeValue();
                String origLocation = at.getNamedItem("location").getNodeValue();
                String srcFileName = origLocation+"/"+name;
                java.io.FileInputStream fIn = new java.io.FileInputStream(srcFileName);
                byte[] data = new byte[fIn.available()];
                fIn.read(data);
                String destFileName = null;
                if (type.equals("descr")) {
                    destFileName = rootPath+name;
                    java.io.OutputStream os = new java.io.FileOutputStream(destFileName);
                    os.write(data);
                    fIn.close();
                    os.close();
                    listOfInstalledFiles.add(destFileName);
                } else if (type.equals("implClass")) {
                    destFileName = servicesImplPathClassFiles+name;
                    destFileName = rootPath+name;
                    java.io.OutputStream os = new java.io.FileOutputStream(destFileName);
                    os.write(data);
                    fIn.close();
                    os.close();
                    listOfInstalledFiles.add(destFileName);                    
                } else if (type.equals("implJar")) {
                    destFileName = servicesImplPathJarFiles+name;        
                    String[] classFiles = CoreBroker.Utils.extractJarFile(srcFileName,servicesImplPathClassFiles);
                    for (int k=0; k<classFiles.length; k++)
                    listOfInstalledFiles.add(classFiles[k]);   
                }
              }
            }            
        }
        catch( Exception ex ) {
            ex.printStackTrace();
        }
        
        String[] arrayOfInstalledFiles = new String[listOfInstalledFiles.size()];
        listOfInstalledFiles.copyInto(arrayOfInstalledFiles);
        return arrayOfInstalledFiles;
    }
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    private void removeServiceFromConfigFile(String serviceName) {
        DocumentBuilderFactory dbf = null;        
        DocumentBuilder db = null;
        Document configDoc = null;
        
        try {
            dbf = DocumentBuilderFactory.newInstance();        
            db = dbf.newDocumentBuilder();
        } catch (javax.xml.parsers.ParserConfigurationException ex) {
            Syslog.error(this,"Error creating install/deploy file ");
        }
        
        try {
            configDoc = db.parse(new java.io.FileInputStream(wsamiConfigXML));
        } catch (org.xml.sax.SAXException ex) {
            Syslog.warning(this,"Error "+wsamiConfigXML+" does not exist.");
        } catch (java.io.FileNotFoundException ex) {
            Syslog.warning(this,wsamiConfigXML+" does not exist.");            
        } catch (java.io.IOException e) {
            Syslog.error(this,"Error reading file "+wsamiConfigXML);
        }    
        
        NodeList serviceNodes = configDoc.getElementsByTagName("service");
        for (int k=0; k<serviceNodes.getLength(); k++) {
            Node serviceNode = serviceNodes.item(k);
            String nameServiceNode = serviceNode.getAttributes().getNamedItem("name").getNodeValue();
            if (nameServiceNode.equals(serviceName)) {
                Node parentServiceNode = serviceNode.getParentNode();
                parentServiceNode.removeChild(serviceNode);
            }
        }

        try {
            java.io.FileOutputStream fos = new java.io.FileOutputStream(wsamiConfigXML);
            CoreBroker.Utils.printDocToStream(configDoc, fos);
        } catch (java.io.FileNotFoundException ex) {
            Syslog.warning(this,"Error writing file "+wsamiConfigXML);
        } 
    }
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    private void removeServiceFromInstalledServicesFile(String serviceName) {
        DocumentBuilderFactory dbf = null;        
        DocumentBuilder db = null;
        Document installDoc = null;
        
        try {
            dbf = DocumentBuilderFactory.newInstance();        
            db = dbf.newDocumentBuilder();
        } catch (javax.xml.parsers.ParserConfigurationException ex) {
            Syslog.error(this,"Error creating install/deploy file ");
        }
        
        try {
            installDoc = db.parse(new java.io.FileInputStream(wsamiInstalledServicesXML));
        } catch (org.xml.sax.SAXException ex) {
            Syslog.warning(this,"Error "+wsamiInstalledServicesXML+" does not exist.");
        } catch (java.io.FileNotFoundException ex) {
            Syslog.warning(this,wsamiInstalledServicesXML+" does not exist.");            
        } catch (java.io.IOException e) {
            Syslog.error(this,"Error reading file "+wsamiInstalledServicesXML);
        }    
            
        NodeList installNodes = installDoc.getElementsByTagName("install");
        for (int k=0; k<installNodes.getLength(); k++) {
            Node installNode = installNodes.item(k);
            String nameInstallNode = installNode.getAttributes().getNamedItem("name").getNodeValue();
            if (nameInstallNode.equals(serviceName)) {
                Node parentInstallNode = installNode.getParentNode();
                parentInstallNode.removeChild(installNode);
            }
        }

        try {
            java.io.FileOutputStream fos = new java.io.FileOutputStream(wsamiInstalledServicesXML);
            CoreBroker.Utils.printDocToStream(installDoc, fos);
        } catch (java.io.FileNotFoundException ex) {
            Syslog.warning(this,"Error writing file "+wsamiInstalledServicesXML);
        } 
    }    
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    private void addServiceToConfigFile(String serviceName, Node serviceNode) {
        DocumentBuilderFactory dbf = null;
        DocumentBuilder db = null;
        Document configDoc = null;
        
        try {
            dbf = DocumentBuilderFactory.newInstance();        
            db = dbf.newDocumentBuilder();
        } catch (javax.xml.parsers.ParserConfigurationException ex) {
            Syslog.error(this,"Error creating install/deploy file ");
        }
        
        boolean wsamiConfigXMLExists = true;
        try {
            configDoc = db.parse(new java.io.FileInputStream(wsamiConfigXML));
        } catch (org.xml.sax.SAXException ex) {
            Syslog.error(this,"Error creating install/deploy file6 ");
        } catch (java.io.FileNotFoundException ex) {
            Syslog.warning(this,wsamiConfigXML +" does not exist.");
            wsamiConfigXMLExists = false;
        } catch (java.io.IOException e) {
            Syslog.warning(this,wsamiConfigXML +" does not exist.");
            wsamiConfigXMLExists = false;
        }
            
        if (!wsamiConfigXMLExists) {
            configDoc = db.newDocument();
            Element root = configDoc.createElement("deploy");

            root.setAttribute("xmlns","http://www-rocq.inria.fr/arles/wsami");
            root.setAttribute("xmlns:xsi","http://www.w3.org/2001/XMLSchema-instance");
            root.setAttribute("xsi:schemaLocation","http://www-rocq.inria.fr/arles/wsami deploy.xsd");
            configDoc.appendChild(root);   
        } 
        
        /*
         *  verify if the service is already defined in config file
         *  and if it is, remove the node from the config document
         *  before adding the new service node definition
         */
        NodeList checkingServiceNodes = configDoc.getElementsByTagName("service");
        for (int k=0; k<checkingServiceNodes.getLength(); k++) {
            Node checkingServiceNode = checkingServiceNodes.item(k);
            String nameServiceNode = checkingServiceNode.getAttributes().getNamedItem("name").getNodeValue();
            if (nameServiceNode.equals(serviceName)) {
                Node parentServiceNode = checkingServiceNode.getParentNode();
                parentServiceNode.removeChild(checkingServiceNode);
            }
        }        

        /*
         *  add the new service node to the config document
         */
        //String serviceName = serviceNode.getAttributes().getNamedItem("name").getNodeValue();                
        Node configServiceNode =configDoc.importNode(serviceNode,true);
        configDoc.getDocumentElement().appendChild(configServiceNode);

        /*
         *  write the new config document to file
         */
        try {
            java.io.FileOutputStream fos = new java.io.FileOutputStream(wsamiConfigXML);
            CoreBroker.Utils.printDocToStream(configDoc, fos);
            if (!wsamiConfigXMLExists) 
                    Syslog.info(this,wsamiConfigXML+" created.");
        } catch (java.io.FileNotFoundException ex) {
            Syslog.error(this,"error creating "+wsamiConfigXML);
        }
    }
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    private Document InStr2Document(java.io.InputStream is, boolean validationRequired) 
        throws CBException {        
        /*
         *  use DOM Parser instead of BuilderFactory because
         *  XML Schema validation is not implemented in 
         *  BuilderDocument (only DTD validation is implemented )
         */           
        org.apache.xerces.parsers.DOMParser domParser = new org.apache.xerces.parsers.DOMParser();
        try {
            domParser.setFeature("http://xml.org/sax/features/validation", true);
            domParser.setFeature("http://apache.org/xml/features/validation/schema", true);
        } catch (org.xml.sax.SAXNotRecognizedException ex) {
            if (validationRequired)
                throw new CBException("XMLparsing01","XML Validation not supported by parser");
        } catch (org.xml.sax.SAXNotSupportedException ex) {
            if (validationRequired)
                throw new CBException("XMLparsing01","XML Validation not supported by parser");
        }
        
        /*
         *  EntityResolver is used to retrieve XSD files 
         *  (for XML Schema validation)
         *  in the directory of the wsami context 
         */
        EntityResolver er = new EntityResolverHandler();            
        domParser.setEntityResolver(er) ;

        DeployErrorHandler errorHandler = new DeployErrorHandler();
        domParser.setErrorHandler (errorHandler);        

        try {
            domParser.parse(new org.xml.sax.InputSource(is));     
        } catch( org.xml.sax.SAXException ex ) {
            throw new CBException("XMLparsing00","Error in parsing XML data");
        } catch( java.io.IOException ex ) {
            throw new CBException("XMLparsing02","Error reading from input");
        }
        
        if (errorHandler.error)
            throw new CBException("XMLparsing00","Error validating XML data: "+errorHandler.error_descr);

        return domParser.getDocument();
    }
  
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    private class DeployErrorHandler implements ErrorHandler {
        boolean error = false;
        String error_descr = null;

        public void warning (SAXParseException e)
            throws SAXException {        
        }

        public void error (SAXParseException e)
                throws SAXException {
            error = true;
            error_descr = e.getMessage();
        }

        public void fatalError (SAXParseException e)
                throws SAXException {
            error = true;    
            error_descr = e.getMessage();
        }
    }
    
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException
    {

        Document deployDoc = null;
        String error = null;

        try {
            try {
                deployDoc = InStr2Document(request.getInputStream(),true);
            } catch (CBException ex) {
                throw new CBException("AdminServlet01",ex.getDescription());
            } catch (IOException ex) {
                throw new CBException("AdminServlet01","Error reading HTTP stream: "+ex.getMessage());
            } 

            String rootNodeName = deployDoc.getDocumentElement().getNodeName();

            if (rootNodeName.equals("deploy")) {                

                /*
                 *  separate deploy and install doc
                 *  for all services in deployDoc
                 *  and install files for all services
                 *
                 */
                HashMap allInstallNodes = new HashMap();
                
                NodeList installNodes = deployDoc.getElementsByTagNameNS("*","install");
                for (int k=0; k<installNodes.getLength(); k++) {
                    
                    // removing install child-node from the
                    // service node in deployDoc ...
                    Node installNode = installNodes.item(k);
                    Node parentInstallNode = installNode.getParentNode();
                    parentInstallNode.removeChild(installNode);
                    
                    // ... the removed install child-node is put
                    // in the separate structure allInstallNodes
                    String serviceName = installNode.getAttributes().getNamedItem("name").getNodeValue();                
                    allInstallNodes.put(serviceName,installNode);
                    
                }
                
                /*
                 *  deploy all services in deployDoc
                 */
                deployServices(deployDoc, allInstallNodes, true);

                /*
                 *  updates CoreBroker DeployedService data
                 *  about installed files
                 *  
                 
                Object[] keys = allInstalledFiles.keySet().toArray();
                for (int i=0; i<keys.length; i++) {                    
                    String[] installedFiles = (String[])allInstalledFiles.get(keys[i]);

                //String serviceName = deployDoc.getElementsByTagNameNS("*","service").item(0).getAttributes().getNamedItem("name").getNodeValue();
                    DeployedService ds = CoreBroker.BrokerWrapper.getDeployedService((String)keys[i]);
                    ds.setFiles(installedFiles);                    
                    addServiceToInstalledServicesFile((String)keys[i], installedFiles);
                }
                 **/
                
            } else if (rootNodeName.equals("undeploy")) {
                /*
                 *  remove all files for each service
                 *  in deployDoc
                 *
                NodeList serviceNodes = deployDoc.getElementsByTagNameNS("*","service");
                for (int k=0; k<serviceNodes.getLength(); k++) {
                    Node serviceNode = serviceNodes.item(k);
                    String serviceName = serviceNode.getAttributes().getNamedItem("name").getNodeValue();
                    
                    removeInstalledFiles(serviceName);
                    removeServiceFromConfigFile(serviceName);
                    removeServiceFromInstalledServicesFile(serviceName);
                }
                 */
                
                /*
                 *  undeploy all services in deployDoc
                 */

                undeployServices(deployDoc);

            } else {
                throw new CBException ("AdminServlet00","Not recognized command in xml file");
            }
        } catch (CBException ex) {
            error = ex.getDescription();
        }

        if (error != null) {
            try {
                Syslog.error(this,error);
                
                response.setContentType("text/html");
                response.sendError(HTTP_ERRORCODE);
                java.io.OutputStream  os = response.getOutputStream();
                os.write(error.getBytes());
                os.flush();
                os.close();
            } catch (java.io.IOException ex) {
                Syslog.error(this,"Error sending error reply: "+error);
            }
        }
    }

        
    public class EntityResolverHandler implements EntityResolver {
       
        public org.xml.sax.InputSource resolveEntity(java.lang.String publicId, java.lang.String systemId) {
            /*
             *  retrieve the name of the .xsd file
             *  and build the complete filename path
             *  in the local filesystem
	     *
             *  the systemId name is in urn format
             *  (ex: file:///home/filename.xsd of file:///c:/home/filename.xsd
	     *  so the separator is always "/" character
             */
  	    String separator ="/";
            int indexSeparator = systemId.lastIndexOf(separator);
            if (indexSeparator == -1) 
                return null;

            String XSDfilename = systemId.substring(indexSeparator+1);        
            String resourceName = contextRealPath.concat(XSDfilename);

            /*
             *  try to create InputSource from xsd filename
             */
            try {
                return new org.xml.sax.InputSource(new java.io.FileInputStream(resourceName)); 
            } catch (java.io.FileNotFoundException ex) {
                Syslog.warning(this,"File "+resourceName+" not found.");
                return null;
            }
        }
    }
    
    /**
    * Method description
    * @param p1
    * @param p2
    * @param p3
    * @return
    */
    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException//, IOException 
	{
	try {
		CoreBroker.BrokerWrapper broker=CoreBroker.BrokerFactory.getServerInstance();
		DeployedService[] deployedServices=broker.getDeployedServices();
    		response.resetBuffer();
 	   	response.setContentType("text/html");
		PrintWriter out=response.getWriter();
		out.println("<HTML>");
		
		//generating HEAD definition
		out.println("<HEAD>");
		out.println("<TITLE>");
		out.println("--wsami project--");
		out.println("</TITLE>");
		out.println("</HEAD>");
		
		//generating BODY definition 
		out.println("<BODY>");
		//generating title
		out.println("<H3><B>");
		out.println("Deployed services information");	
		out.println("</B></H3>");
                
		for (int k=0; k<deployedServices.length; k++) {
			DeployedService deployedService=deployedServices[k];
			//######################
			//Information retrieving
			//######################
			
			//Retrieving endpoint
			String endPoint=deployedService.getEndPoint();
			
                        // retrieving service name
                        String serviceName = deployedService.getServiceName();
                        
                        // retrieving instance URL
                        String instanceURL = deployedService.getInstanceURL();
                        
			//retrieving serviceInfo DOM Document
			Document serviceInfoDoc=deployedService.getServiceInfo();
			
			//String serviceInfoXMLText=null;
			//try {
			//	OutputFormat format=new OutputFormat(serviceInfoDoc);
			//	StringWriter stringOut=new StringWriter();
			//	XMLSerializer serial=new XMLSerializer(stringOut,format);
			//	serial.asDOMSerializer();
			//	serial.serialize(serviceInfoDoc.getDocumentElement());
			//	serviceInfoXMLText=stringOut.toString();
			//	
			//}
			//catch (Exception e) {
			//}
			
			NodeList serviceNodeList=serviceInfoDoc.getElementsByTagName("service");

			String/*[]*/ serviceNameWSDD=null; //=new String[serviceNodeList.getLength()];

			//to avoid to parse the document I use a heuristic method to dimension these arrays
			int numMaxParameter=serviceInfoDoc.getElementsByTagName("parameter").getLength();
			
			int numMaxOperation=serviceInfoDoc.getElementsByTagName("operation").getLength();
			
			String/*[]*/[] serviceParameterName=new String/*[serviceNodeList.getLength()]*/[numMaxParameter];
			String/*[]*/[] serviceParameterValue=new String/*[serviceNodeList.getLength()]*/[numMaxParameter];
			String/*[]*/[] serviceOperationName=new String/*[serviceNodeList.getLength()]*/[numMaxOperation];

			for (int s=0;s<serviceNodeList.getLength();s++) {
				Node serviceItem=serviceNodeList.item(s);
				NamedNodeMap serviceAttr=serviceItem.getAttributes();
				Node serviceNameWSDDAttr=serviceAttr.getNamedItem("name");                                
				serviceNameWSDD/*[s]*/=serviceNameWSDDAttr.getNodeValue();
				
				NodeList serviceChildren=serviceItem.getChildNodes();
				int countPar=0;
				int countOp=0;
				for (int n=0;n<serviceChildren.getLength();n++) {
					Node serviceChild=serviceChildren.item(n);
					if (serviceChild.getNodeName().equals("parameter")) {
						NamedNodeMap parameterAttr=serviceChild.getAttributes();
						Node parameterNameAttr=parameterAttr.getNamedItem("name");
						serviceParameterName/*[s]*/[countPar]=parameterNameAttr.getNodeValue();
						Node parameterValueAttr=parameterAttr.getNamedItem("value");
						serviceParameterValue/*[s]*/[countPar]=parameterValueAttr.getNodeValue();						
						countPar++;
					}
				}	
				
				for (int n=0;n<serviceChildren.getLength();n++) {
					Node serviceChild=serviceChildren.item(n);
					if (serviceChild.getNodeName().equals("operation")) {
						NamedNodeMap operationAttr=serviceChild.getAttributes();
						Node operationNameAttr=operationAttr.getNamedItem("name");
						serviceOperationName/*[s]*/[countOp]=operationNameAttr.getNodeValue();
						countOp++;
					}
				}
					
				
			}
			
			//retrieving WSAMIAbstractDocument
			WSAMIAbstractDocument WSAMIAbDoc=deployedService.getWSAMIAbstractDocument();
			
			//retrieving ConnectorQoS
			String[] connectors=WSAMIAbDoc.getConnectorQoS();
			
			//retrieving targetNamespace
			String targetNamespace=WSAMIAbDoc.getTargetNamespace();			
			
			//retrieving conversation
			String conversation=WSAMIAbDoc.getConversation();
			
			//retrieving interface
			String interfaceInfo=WSAMIAbDoc.getInterface();
			
			//retrieving WSAMICustomizerDocument
			WSAMICustomizerDocument WSAMICuDoc=deployedService.getWSAMICustomizerDocument();
			
			//retrieving QoSCriterion
			String[] QoSCriterion=null;
			if (WSAMICuDoc!=null)		
				QoSCriterion=WSAMICuDoc.getQoSCriterion();
			
			//retrieving local
			String local=null;
			if (WSAMICuDoc!=null)
				local=WSAMICuDoc.getLocal();
			
			//retrieving remote
			String remote=null;
			if (WSAMICuDoc!=null)
				remote=WSAMICuDoc.getRemote();
				
			//retrieving service WSAMIServiceDocumetn
			WSAMIServiceDocument WSAMISerDoc=deployedService.getWSAMIServiceDocument();
			
			//retrieving Abstract
			String abstractInfo=WSAMISerDoc.getAbstract();
			
			//retrieving Concrete=WSAMISerDoc.getConcrete();
			String concrete=WSAMISerDoc.getConcrete();
			
			//retrieving NReqServices
			int nReqServices=WSAMISerDoc.getNReqServices();
			nReqServices=-1;	
			 
			//###############
			//Code generation
			//###############
			out.println("<BR>");
			out.println("<H3>");
			int num=k+1;
			out.println("Service n."+num);
			out.println("</H3>");					
			out.println("<BR>");
                        
			//generating code for displaying service name			
			if (serviceName/*[0]*/!=null) {
				out.println("<H4>");
				out.println("Service name: ");
				out.println(serviceName/*[0]*/);
				out.println("</H4>");
			} else {
				out.println("<H4>");
				out.println("Service name: ");
				out.println("information not available");
				out.println("</H4>");
			}

			//if (instanceURL != null) {
				out.println("<H4>");
				out.println("Instance URL: ");
				out.println(instanceURL);
				out.println("</H4>");
			//}                         
                        
			//generating some info about the service extracted from the parameter children of service
			for (int f=0;f<numMaxParameter;f++) {
				if (serviceParameterName/*[0]*/[f]!=null) {
					out.println("<H4>");
					out.println(serviceParameterName/*[0]*/[f]);
					out.println(": ");
					out.println(serviceParameterValue/*[0]*/[f]);
					out.println("</H4>");
				}
			}
			
			//generating a list of available operations
			out.println("<H4>");
			out.println("Operations: ");
			out.println("</H4>");
			for (int f=0;f<numMaxOperation;f++) {
				if (serviceOperationName/*[0]*/[f]!=null) {
					out.println("<H4>");
					out.println("-->");
					out.println(serviceOperationName/*[0]*/[f]);
					out.println("</H4>");
				}
			}
				
				
			//generating code for displaying endpoint			
			out.println("<H4>");
			out.println("Endpoint: ");
			out.println(endPoint);
			out.println("</H4>");
							
			out.println("<BR>");
			
			//generating information extracted from WSAMIAbstractDocument
			out.println("<H3>");
			out.println("WSAMI Abstract Document Information");
			out.println("</H3>");
			
			if (targetNamespace!=null) {
				out.println("<H4>");
				out.println("Target namespace: ");
				out.println(targetNamespace);
				out.println("</H4>");
			} else {
				out.println("<H4>");
				out.println("Target namespace: ");
				out.println("information not available");
				out.println("</H4>");
			}
						
			
			if (connectors!=null) {
				for (int p=0;p<connectors.length;p++) {
					out.println("<H4>");
					out.println("ConnectorQoS "+p+":");
					out.println(connectors[p]);
					out.println("</H4>");
				}
				
			} else {
					out.println("<H4>");
					out.println("ConnectorQoS: ");
					out.println("information not available");
					out.println("</H4>");
			
			}
			
						
			if (interfaceInfo!=null) {
				out.println("<H4>");
				out.println("Interface: ");
				out.println(interfaceInfo);
				out.println("</H4>");
			} else {
				out.println("<H4>");
				out.println("Interface: ");
				out.println("information not available");
				out.println("</H4>");
			}
			
			
			if (conversation!=null) { 						
				out.println("<H4>");
				out.println("Conversation: ");
				out.println(conversation);
				out.println("</H4>");
			} else {
				out.println("<H4>");
				out.println("Conversation: ");
				out.println("information not available");
				out.println("</H4>");
			}
										
		
			out.println("<BR>");
			
			//generating information extracted from WSAMICustomizerDocument
			out.println("<H3>");
			out.println("WSAMI Customizer Document Information");
			out.println("</H3>");
			
			if (local!=null) { 						
				out.println("<H4>");
				out.println("Local: ");
				out.println(local);
				out.println("</H4>");
			} else {
				out.println("<H4>");
				out.println("Local: ");
				out.println("information not available");
				out.println("</H4>");
			}
					
			
			if (remote!=null) { 						
				out.println("<H4>");
				out.println("Remote: ");
				out.println(remote);
				out.println("</H4>");
			} else {
				out.println("<H4>");
				out.println("Remote: ");
				out.println("information not available");
				out.println("</H4>");
			}
			
			
			if (QoSCriterion!=null) 
				for (int p=0;p<QoSCriterion.length;p++) {
					out.println("<H4>");
					out.println("QoS Criterion "+p+":");
					out.println(QoSCriterion[p]);
					out.println("</H4>");
			} else {
			 		out.println("<H4>");
					out.println("QoS Criterion: ");
					out.println("information not available");
					out.println("</H4>");
			}
   		
			out.println("<BR>");
			
			
			//generating information extracted from WSAMIServiceDocument
			out.println("<H3>");
			out.println("WSAMI Service Document Information");
			out.println("</H3>");
			if (abstractInfo!=null) {
				out.println("<H4>");
				out.println("Abstract: ");
				out.println(abstractInfo);
				out.println("</H4>");
			} else {
				out.println("<H4>");
				out.println("Abstract: ");
				out.println("no information available");
				out.println("/H4");
			}
			
			if (concrete!=null) {
				out.println("<H4>");
				out.println("Concrete: ");
				out.println(concrete);
				out.println("</H4>");
			} else {
				out.println("<H4>");
				out.println("Concrete: ");
				out.println("no information available");
				out.println("</H4>");
			}
			
			if (nReqServices!=-1) {
				out.println("<H4>");
				out.println("NReqServices: ");
				out.println(""+nReqServices);
				out.println("</H4>");
			} else {
				out.println("<H4>");
				out.println("NReqServices: ");
				out.println("no information available");
				out.println("</H4");
			}
			
			
			
			//separating services information
			out.println("<HR/>");	
		}
		
		//closing BODY tag
		out.println("</BODY>");
		//closing HTML tag
		out.println("</HTML>");
		out.close();
	}
	catch (Exception e) {
		//exception handling...insert some code
		e.printStackTrace();
		
	}

    }//end do get

    
}
