package CoreBroker;

import CoreBroker.CBException;
import java.util.Properties;
import java.io.InputStream;

public class BrokerFactory {
    
    private static BrokerWrapper _instanceClient = null;
    private static BrokerWrapper _instanceServer = null;
    private static String _brokerWrapperName = null;
    private static String DEFAULT_BROKERVALUE = "CoreBroker.Axis.AxisBrokerWrapper";

    
    
    public static void setFactory(String brokerWrapperName) 
            /*throws CBException*/ {

       /*
        *   create an instance of Server Broker Wrapper
        */
                
        // check input value        
        if (brokerWrapperName != null) {
            _brokerWrapperName = brokerWrapperName;
            return;            
        }
                
        // check system property BrokerFactory
        String factory = System.getProperty("BrokerFactory");        
        if (factory != null) {
            _brokerWrapperName = factory;
            return;            
        }
        
        // check in property file BrokerFactory.properties
        ClassLoader c = Thread.currentThread().getContextClassLoader();
        InputStream is = c.getResourceAsStream("BrokerFactory.properties");        
        Properties props = new Properties();
        try {
            props.load(is);
        } catch (java.io.IOException ex) { 
            ex.printStackTrace();
        }        
        factory = props.getProperty("BrokerFactory");
        if (factory != null) {
            _brokerWrapperName = factory;
            return;            
        }
                
        _brokerWrapperName = DEFAULT_BROKERVALUE;
    }
    
    
    public static BrokerWrapper getServerInstance() 
            throws CBException {

         if (_instanceServer == null) {
           try {
               if (_brokerWrapperName == null)
                    setFactory(null);
                Class c = java.lang.Class.forName(_brokerWrapperName);
                Class[] parameterTypes = new Class[] {};
                java.lang.reflect.Method meth = c.getMethod("makeServerInstance", parameterTypes);
                Object[] arguments = new Object[] {};
                _instanceServer = (CoreBroker.BrokerWrapper)meth.invoke(null,arguments);            
            } catch (java.lang.ClassNotFoundException ex) {
                throw new CBException("broker00","Class "+_brokerWrapperName+" not found");                
            } catch (java.lang.NoSuchMethodException ex) {
                throw new CBException("broker00","Class "+_brokerWrapperName+" is not a BrokerWrapper implementation");
            } catch (java.lang.IllegalAccessException ex) {
                throw new CBException("broker00","Error accessing class "+_brokerWrapperName);
            } catch (java.lang.reflect.InvocationTargetException ex) {
                throw new CBException("broker00","Error accessing class "+_brokerWrapperName);
            } 
        } 
            return _instanceServer;
    }
        
    
    public static BrokerWrapper getClientInstance() 
            throws CBException {
        
        if (_instanceClient == null) {        
            try {
               if (_brokerWrapperName == null)
                    setFactory(null);                
                Class c = java.lang.Class.forName(_brokerWrapperName);
                Class[] parameterTypes = new Class[] {};
                java.lang.reflect.Method meth = c.getMethod("makeClientInstance", parameterTypes);
                Object[] arguments = new Object[] {};
                _instanceClient = (CoreBroker.BrokerWrapper)meth.invoke(null,arguments);            
            } catch (java.lang.ClassNotFoundException ex) {
                Syslog.error(BrokerFactory.class,"Class "+_brokerWrapperName+" not found");
                throw new CBException("broker00","Class "+_brokerWrapperName+" not found");
            } catch (java.lang.NoSuchMethodException ex) {
                Syslog.error(BrokerFactory.class,"Class "+_brokerWrapperName+" is not a BrokerWrapper implementation");
                throw new CBException("broker00","Class "+_brokerWrapperName+" is not a BrokerWrapper implementation");
            } catch (java.lang.IllegalAccessException ex) {
                Syslog.error(BrokerFactory.class,"Error accessing class "+_brokerWrapperName);
                throw new CBException("broker00","Error accessing class "+_brokerWrapperName);
            } catch (java.lang.reflect.InvocationTargetException ex) {
                Syslog.error(BrokerFactory.class,"Error2 accessing class "+_brokerWrapperName);
                throw new CBException("broker00","Error accessing class "+_brokerWrapperName);
            } 
        }
        return _instanceClient;
    }
    
}
