package CoreBroker;

import org.w3c.dom.*;
import WSAMI.*;
import java.util.*;
import CoreBroker.Utils;
import CoreBroker.IFUtils;

public abstract class BrokerWrapper {
    
    static protected HashMap     customizerConfig;
   
    static private   HashMap     deployedServices;       // hashmap of DeployedService objects  
    static private   Document    serverConfiguration;    // document for server-specific configuration
    static protected Document    deploymentConfiguration;// document generated merging server-specific 
                                                        // configuration and services configuration
                                                        // retrieved in hashmap deployedservices
   
    static final String ND_WAU = "http://localhost:8080/wsami/NDAbstract.wsami";                                                            
   

    /*  BEGIN
     *  modification to solve bug getClientInstance / getServerInstance
     *  returning null if an instance already exists
     */
    static protected BrokerWrapper _selfServerSide; 
    static protected BrokerWrapper _selfClientSide; 
   
    protected String fileSeparator;
    
    protected static String BROKERWRAPPER_SERVERSIDE = "server";
    protected static String BROKERWRAPPER_CLIENTSIDE = "client";
    //protected static String brokerSide = null;

    /*  END
     *  modification to solve bug getClientInstance / getServerInstance
     *  returning null if an instance already exists
     */
    
    protected String wsamiBinDir;
    protected String URAddr = null;
    
    /*
     * Creates a new instance of Broker 
     */
    protected BrokerWrapper() 
    {      
        deployedServices = new HashMap();
        customizerConfig = new HashMap();
        serverConfiguration = null;
        deploymentConfiguration = null;
        
        java.util.Properties props = System.getProperties();
        fileSeparator = props.getProperty("file.separator"); 
        
        String logLevelStr = props.getProperty("log.level");
        try {
            if (logLevelStr != null) {
                int logLevel = new Integer(logLevelStr).intValue();
                Syslog.setLogLevel(logLevel);
                System.out.println("loglevel is "+logLevel);
            } else
                Syslog.debug(this,"log.level property not defined");
        } catch (NumberFormatException ex) {
            Syslog.debug(this,"log.level property value not valid");
        }        
    }
   
    protected static BrokerWrapper makeServerInstance() {return null; };
    protected static BrokerWrapper makeClientInstance() {return null; };

    
    
    /* 
     *  Methods used for server-specific configuratio
     */
   
    protected Document getServerConfig () {
       return (Document)serverConfiguration.cloneNode(true);
    }

    protected void setServerConfig (Document _serverConfiguration) {
       serverConfiguration = (Document)_serverConfiguration.cloneNode(true);
       refreshDeploymentConfiguration();
    }
    
    
    /*
     *  Methods used for service deployment
     */

    public Document getDeploymentConfiguration() {
        return deploymentConfiguration;
    } 
    
    protected abstract void refreshDeploymentConfiguration();   

    public static DeployedService[] getDeployedServices() {
        Object[] dsObj = deployedServices.values().toArray();
        DeployedService[] ds = new DeployedService[dsObj.length];
        for (int i=0; i<ds.length; i++)
            ds[i] = (DeployedService)dsObj[i];
        return ds;
    }
    
    public void undeployService(String serviceName /*Node service*/) 
            throws CBException {
        //String WAU = getTagValue(service, "wau"); 
        if (deployedServices.remove(serviceName) == null)
            throw new CBException("deploy15","Error undeploying: the service was not deployed");
        refreshDeploymentConfiguration();
    }

    public abstract NDService.NDPortType getNDStub(String ndAddress) 
        throws CBException;
    
    /*
     * @path = the directory where are all the xsd and wsami files
     * @baseURLService = the http URL used to reference services (http://..../wsami/services
     * @context = the name of the Web context that owns the wsami implementation
     */    
    public boolean verify(Node service, String path, String context, String baseURLServices) 
            throws CBException {
                
        /*
         * Verify that file WAD, WSD, WCD exist on the local filesystem
         */        
        String[] wsami = new String[3];        
        wsami[0] = BrokerWrapper.getTagValue(service, "wad");
        wsami[1] = BrokerWrapper.getTagValue(service, "wsd"); 
        wsami[2] = BrokerWrapper.getTagValue(service, "wcd"); 

        CoreBroker.IFUtils ifUtils = new CoreBroker.IFUtils();
        
        if (wsami[0] == null)
            throw new CBException("deployverify00","WSAMI Abstract document not defined in XML deployment file");
        if (wsami[1] == null)
            throw new CBException("deployverify00","WSAMI Service document not defined in XML deployment file");

        for (int i=0; i<wsami.length; i++) {
            if (wsami[i] != null /*possible only for i==2 (customizer)*/)
                try {                
                    String file = path.concat(wsami[i]);
                    java.io.FileInputStream is = new java.io.FileInputStream(new java.io.File(file));
                    int rb = is.read();
                    if (rb == -1)
                        throw new java.io.IOException();
                } catch (java.io.FileNotFoundException ex) {
                    throw new CBException("deployverify00","WSAMI file "+wsami[i]+" not found.");
                } catch (java.io.IOException ex) {
                    throw new CBException("deployverify00","WSAMI file "+wsami[i]+" not found.");
                }
        }

        /* 
         * verify that the Service Interface.wsdl file retrieved from WAD file
         *          exist on local filesystem or on the remote http address    
         */ 

        WSAMIAbstractDocument wad = new WSAMIAbstractDocument(path, path+wsami[0]);       
        String wsdlInterface = wad.getInterface();
        try {

            java.net.URL urlWSDLInterface = new java.net.URL(wsdlInterface);
            /*
             * if the WSDL interface document is on local machine : 
             *     trasform the http address retrieved in WSAMI Abstract Document
             *      into absolute path on local filesystem 
             * else keep the http address
             */
            if (ifUtils.isLocalAddress(urlWSDLInterface.getHost())) {
                wsdlInterface = ifUtils.localURI2localfile(urlWSDLInterface,path,context);
                java.io.FileInputStream is = new java.io.FileInputStream(new java.io.File(wsdlInterface));
                int rb = is.read();
                if (rb == -1)
                    throw new java.io.IOException();

            } else {
                /*
                 *  to implement the case of a remote wsdl interface
                 *
                 *
                 */
            }         
        } catch (java.io.FileNotFoundException ex) {
            throw new CBException("deployverify01","WSDL interface file "+wsdlInterface+" not found.");
        } catch(java.net.MalformedURLException ex) {
            throw new CBException("deployverify01","Invalid address "+wsdlInterface+" for WSDL interface.");            
        } catch (java.io.IOException ex) {
            throw new CBException("deployverify01","WSDL interface file "+wsdlInterface+" not found.");
        }
        
        /* 
         * verify that the Service Concrete.wsdl file retrieved from WSD file
         *          exist on local filesystem or on the remote http address    
         */                
        WSAMIServiceDocument wsd = new WSAMIServiceDocument(path, path+wsami[1]);        
        String wsdlConcrete = wsd.getConcrete();
        boolean isWSDLConcreteLocal = true;
        try {
            java.net.URL urlWSDLConcrete = new java.net.URL(wsdlConcrete);
            /*
             * if the WSDL concrete document is on local machine : 
             *     trasform the http address retrieved in WSAMI Service Document
             *      into absolute path on local filesystem 
             * else keep the http address
             */
            if (ifUtils.isLocalAddress(urlWSDLConcrete.getHost())) {
                isWSDLConcreteLocal = true;
                wsdlConcrete = ifUtils.localURI2localfile(urlWSDLConcrete,path,context);
                java.io.FileInputStream is = new java.io.FileInputStream(new java.io.File(wsdlConcrete));
                int rb = is.read();
                if (rb == -1)
                    throw new java.io.IOException();
            } else {
                isWSDLConcreteLocal = false;
                java.io.InputStream is = urlWSDLConcrete.openStream();
                int rb = is.read();
                if (rb == -1)
                    throw new java.io.IOException();
            }
        } catch (java.io.FileNotFoundException ex) {
            throw new CBException("deployverify02","WSDL concrete file "+wsdlConcrete+" not found.");
        } catch(java.net.MalformedURLException ex) {
            throw new CBException("deployverify02","Invalid address "+wsdlConcrete+" for WSDL concrete.");            
        } catch (java.io.IOException ex) {
            throw new CBException("deployverify02","WSDL concrete file "+wsdlConcrete+" not found.");
        }
        
        
        /*
         *  verify that WAU in wsd is the same value specified in deploy.xml
         */
        if (!wsd.getAbstract().equals(BrokerWrapper.getTagValue(service, "wau")))
            throw new CBException("deployverify03",
                "Error: WAU value in deploy XML file is "+BrokerWrapper.getTagValue(service, "wau")
                +"and in WSAMI Service document is "+wsd.getAbstract());        
        
        
        /*
         *  verify between wcd and wad for QOS
         */
        String[] qosCritWAD = wad.getConnectorQoS();
        if (qosCritWAD != null && qosCritWAD.length != 0) {
            
            String wadCritList = "(";
            for (int i=0; i<qosCritWAD.length-1; i++)
                wadCritList += qosCritWAD[i]+", ";
            wadCritList += qosCritWAD[qosCritWAD.length-1]+")";
            
            if (wsami[2] == null)
                throw new CBException("deployverify04",
                    "Error: QoS criteria defined in WSAMI Abstract Document "+
                    wadCritList+" and no WSAMI Customizer Document");
            else {
                WSAMICustomizerDocument wcd = new WSAMICustomizerDocument(path, path+wsami[2]);        
                String[] qosCritWCD = wcd.getQoSCriterion();
                if (qosCritWCD == null)
                    throw new CBException("deployverify04",
                        "Error: QoS criteria defined in WSAMI Abstract Document "+
                        wadCritList+" and no QoS criteria defined in WSAMI Customizer Document");
                else {      
                    for (int i=0; i<qosCritWAD.length; i++) {
                        boolean definedInWADandNotInWCD = true;                    
                        for (int j=0; j<qosCritWCD.length; j++)
                            if (qosCritWAD[i].equals(qosCritWCD[j]))
                                definedInWADandNotInWCD = false;
                        if (definedInWADandNotInWCD) {
                            
                            String wcdCritList = "(";
                            for (int k=0; k<qosCritWCD.length-1; k++)
                                wcdCritList += qosCritWCD[k]+", ";
                            wcdCritList += qosCritWCD[qosCritWCD.length-1]+")";

                            throw new CBException("deployverify04",
                                "Error: QoS criteria defined in WSAMI Abstract Document "+
                                "for connectors "+ wadCritList+" does not match QoS "+
                                "criteria defined in WSAMI Customizer Document"+
                                wcdCritList);
                        }
                    }
                }
            }
        } else {               
            if (wsami[2] != null) {

                WSAMICustomizerDocument wcd = new WSAMICustomizerDocument(path, path+wsami[2]);        
                String[] qosCritWCD = wcd.getQoSCriterion();
                
                String wcdCritList = "(";
                for (int i=0; i<qosCritWCD.length-1; i++)
                    wcdCritList += qosCritWCD[i]+", ";
                wcdCritList += qosCritWCD[qosCritWCD.length-1]+")";
                
                throw new CBException("deployverify04",
                    "Error: QoS criteria defined in WSAMI Customizer Document "+
                    wcdCritList+" and no Connector QoS defined in WSAMI Abstract Document");
            }
        }
        
        
        /*
         *  verify endpoint in WSDL concrete and real 
         *  endpoint from Axis deployment
         */
        WSDLConcrete wsdlDoc;
        java.io.InputStream is = null;
        try {
            if (isWSDLConcreteLocal) 
                is = new java.io.FileInputStream(wsdlConcrete);
            else
                is = new java.net.URL(wsdlConcrete).openStream(); 
        } catch (java.io.FileNotFoundException ex) {
            throw new CBException("deployverify02","WSDL concrete file "+wsdlConcrete+" not found.");
        } catch(java.net.MalformedURLException ex) {
            throw new CBException("deployverify02","Invalid address "+wsdlConcrete+" for WSDL interface.");            
        } catch (java.io.IOException ex) {
            throw new CBException("deployverify02","WSDL concrete file "+wsdlConcrete+" not found.");
        }
        wsdlDoc = new WSDLConcrete(path, is);
            
        if (!ifUtils.isSameURL(wsdlDoc.getLocation(),            
                        baseURLServices+"/"+wsdlDoc.getPortName()))
            throw new CBException("deployverify05",
                    "Service location in WSDL concrete file is "+wsdlDoc.getLocation()
                    +" while in deploy XML file is "+baseURLServices+"/"+wsdlDoc.getPortName());
        
        
        // verify if implementation service classes exist
        // ...........
        
        return true;
    }
    
    public abstract void deployService(Node service, String path, String context)
        throws CBException;
    
    protected void deploy(String serviceName, String endPoint, String XSDfilesPath, String WSAMIAbstractFileName, 
            String WSAMIServiceFileName, String WSAMICustomizerFileName, Document serviceInfo, String instanceURL) 
                throws CBException {
        
        WSAMIAbstractDocument dA = null;
        if (WSAMIAbstractFileName != null)
            try {
                dA = new WSAMIAbstractDocument(XSDfilesPath, WSAMIAbstractFileName);           
            } catch (CBException ex) {
                throw new CBException("deploy08",
                    "Error Creating WSAMI Abstract for Service."+ex.getDescription());
            }
 
        WSAMIServiceDocument dS = null;
        if (WSAMIServiceFileName != null)
            try {
                dS = new WSAMIServiceDocument(XSDfilesPath, WSAMIServiceFileName);
            } catch (CBException ex) {
                throw new CBException("deploy08",
                    "Error Creating WSAMI Service for Service."+ex.getDescription());
            }

        WSAMICustomizerDocument dC = null;
        if (WSAMICustomizerFileName != null)
            try {
                dC = new WSAMICustomizerDocument(XSDfilesPath, WSAMICustomizerFileName);                
            } catch (CBException ex) {
                throw new CBException("deploy08",
                    "Error Creating WSAMI customizer for Service."+ex.getDescription());
            }
        deployedServices.put(serviceName, new DeployedService(serviceName, endPoint, dA, dS, dC, serviceInfo, instanceURL));
        refreshDeploymentConfiguration();
    }

    public static boolean isServiceDeployed(String name) {       
        DeployedService ds = (DeployedService)deployedServices.get(name);
        return (ds!=null);
    }    

    public static DeployedService getDeployedService(String name) {
        DeployedService ds = (DeployedService)deployedServices.get(name);
        return ds;
    }
    
    
    public static WSAMIAbstractDocument getWAD_DeployedService(String name) {
        DeployedService ds = (DeployedService)deployedServices.get(name);
        if (ds != null)
            return ds.getWSAMIAbstractDocument();
        else
            return null;
    }
   
    public static WSAMIServiceDocument getWSD_DeployedService(String name) {
        DeployedService ds = (DeployedService)deployedServices.get(name);
        if (ds != null)
            return ds.getWSAMIServiceDocument();
        else 
            return null;
    }

    public static WSAMICustomizerDocument getWCD_DeployedService(String name) {
        DeployedService ds = (DeployedService)deployedServices.get(name);
        if (ds != null)
            return ds.getWSAMICustomizerDocument();
        else 
            return null;           
    }    


    
    /*****************************************
     *  Methods used for Customization       *
     *****************************************/
    private class CustomizerEntry
    {
        String customizerSessionID;
        String LC;
        String RC;
        String WS;
        
        public CustomizerEntry (String sID, String LC, String RC, String WS)
        {
            this.customizerSessionID= sID; this.LC = LC; this.RC = RC; this.WS = WS;
        }         
    } 
    
    public String setCustomizerChain(String LC, String RC, String WS)
    {
        // generate CustomizerSessionID with random + IP address
        String sID = null;
	/*
        try {
            byte[] addr = java.net.InetAddress.getLocalHost().getAddress();
            long nl = addr[0]+addr[1]*256+addr[2]*256*256+addr[3]*256*256*256;
            Random rand = new Random(new Date().getTime());
            int n = rand.nextInt(Integer.MAX_VALUE);
            nl += n*(long)256*256*256*256;
            sID = String.valueOf(nl);
        }catch (Exception e){
            e.printStackTrace();
        }
	*/
	sID = CoreBroker.Utils.getLocalHostRandomString();
        customizerConfig.put(sID, new CustomizerEntry(sID, LC, RC, WS));
        return LC+"?"+sID;
    }

    public void setCustomizerChain_LC(String sID, String RC, String WS)
    {
        customizerConfig.put(sID, new CustomizerEntry(sID, null, RC, WS));
    }

    public void setCustomizerChain_RC(String sID, String WS)
    {
        customizerConfig.put(sID, new CustomizerEntry(sID, null, null, WS));
    }

    public boolean isCustomizer(String sID)
    {
        CustomizerEntry cust =(CustomizerEntry)(customizerConfig.get(sID));
        return (cust != null);
    }
    
    public String getCustomizerRC(String sID)
    {
        CustomizerEntry cust =(CustomizerEntry)(customizerConfig.get(sID));
        return cust.RC;
    }
    
    public String getCustomizerWS(String sID)
    {
        CustomizerEntry cust =(CustomizerEntry)(customizerConfig.get(sID));
        return cust.WS;
    } 

    public void setCustID_and_Chain_FromHeader(javax.xml.soap.SOAPMessage msgSOAP, Object obj)
    {
        String rcValue = null;
        String wsValue = null;
        String sIDValue = null;
        
        try 
        {
            javax.xml.soap.SOAPEnvelope se = msgSOAP.getSOAPPart().getEnvelope();            
            javax.xml.soap.SOAPHeader  sh = se.getHeader();
            java.util.Iterator it = sh.getChildElements();

            while (it.hasNext()) 
            {
                javax.xml.soap.SOAPElement p = (javax.xml.soap.SOAPElement)it.next();
                java.util.Iterator it1 = p.getAllAttributes();
                while (it1.hasNext()) 
                {
                    javax.xml.soap.Name attName = (javax.xml.soap.Name)it1.next();
                    if (attName.getLocalName().equals("RC"))
                        rcValue = p.getAttributeValue(attName);
                    else if (attName.getLocalName().equals("WS"))
                        wsValue = p.getAttributeValue(attName);
                    else if (attName.getLocalName().equals("customizerSessionID"))
                        sIDValue = p.getAttributeValue(attName);

                }                
            }

            // call object's BasicCustomizer Interface's method setCustomizerSessionID
            Class[] parameterTypes = new Class[] {String.class};
            java.lang.reflect.Method meth = BasicCustomizer.class.getMethod("setCustomizerSessionID", parameterTypes);
            Object[] arguments = new Object[] {sIDValue};
            meth.invoke(obj,arguments);

            // set the customizer chain in the local instance of the Core Broker
            if (rcValue!= null && wsValue != null) 
            {
               setCustomizerChain_LC(sIDValue, rcValue, wsValue);
            } else if (rcValue != null)
               setCustomizerChain_RC(sIDValue, wsValue);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public void addCustID_and_ChainToHeader(javax.xml.soap.SOAPMessage msgSOAP, String customizerSessionID)
    {
        try
        {                
            String rcEP = getCustomizerRC(customizerSessionID);
            String wsEP = getCustomizerWS(customizerSessionID);

            Syslog.debug(this,"Cust chain: "+rcEP+wsEP);

            javax.xml.soap.SOAPEnvelope se = msgSOAP.getSOAPPart().getEnvelope();            
            javax.xml.soap.Name nameRC = se.createName("RC_WS");

            javax.xml.soap.SOAPHeader  sh = se.getHeader();
            javax.xml.soap.SOAPHeaderElement shel = sh.addHeaderElement(nameRC);
            shel.addAttribute(se.createName("RC"),rcEP);
            shel.addAttribute(se.createName("WS"),wsEP);
            shel.addAttribute(se.createName("customizerSessionID"),customizerSessionID);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
                
    }

    ///////////////////////////////////////////////////////
    //////////////     static methods      ///////////////
    //////////////////////////////////////////////////////
    
    public static String getLocalNDAddr() throws CBException {
        /*
        if (brokerSide.equals(BROKERWRAPPER_CLIENTSIDE))
            throw new CBException("getlocalNDaddr00","getLocalNDAddr() can be referenced only on Broker Server Side");        
          */
        
        if (_selfServerSide == null)
            throw new CBException("getlocalNDaddr00","getLocalNDAddr() can be referenced only on Broker Server Side");        
        
        DeployedService ds = (DeployedService)deployedServices.get("ND");
        return ds.getEndPoint();
    }
    
    public static void list()
    {
      Object[] deployedList = deployedServices.values().toArray();
      Object[] keys = deployedServices.keySet().toArray();
      
        Syslog.writelog("SIZE="+      deployedList.length);
      for (int i=0; i<deployedList.length; i++)
      {
          try{
            DeployedService ds = (DeployedService)deployedList[i];
            WSAMIAbstractDocument dA = ds.getWSAMIAbstractDocument();
            WSAMIServiceDocument dS = ds.getWSAMIServiceDocument();
            WSAMICustomizerDocument dC = ds.getWSAMICustomizerDocument();
            WSAMIServiceDocument_RequestService[] reqServ = dS.getReqServices();

            Syslog.writelog("WAU for Service ="+keys[i].toString());
            Syslog.writelog("\nDeployed Service\n\n" + /*dA.getName() + "\n" + */dA.getInterface() + "\n" + dA.getConversation() + "\n");
            Syslog.writelog(/*"\n" + dS.getName() + */"\n" + dS.getAbstract() + "\n" + dS.getConcrete());
            for (int j=0; j<reqServ.length; j++)
               Syslog.writelog(reqServ[j].toString());
            Syslog.writelog(/*"\n" + dC.getName() + */"\n" + dC.getLocal() + "\n" + dC.getRemote() + "\n");
          } catch (Exception ex) {
              //ex.printStackTrace();
          }
      }
    }

    
    public static String getTagValue(Node service, String tagName)
    {
       NodeList serviceDef = service.getChildNodes();
       String name = null;

       for (int i=0; i<serviceDef.getLength(); i++)
       {
           Node n = serviceDef.item(i);
           if (n.getNodeName().equals(tagName)) 
               name = n.getAttributes().getNamedItem("name").getNodeValue();
       }
       return name;       
    }
    
    public static String getWSDLfromWSU (String wsu)     
    {
        try
        {
            javax.xml.parsers.DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
            javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
           //org.w3c.dom.Document doc = org.apache.axis.utils.XMLUtils.newDocument(wsu);       
            org.w3c.dom.Document doc = db.parse(wsu);            
           
           org.w3c.dom.Node wsdlConcrete = doc.getElementsByTagName("Concrete").item(0);
           org.w3c.dom.NamedNodeMap attr = wsdlConcrete.getAttributes();
           String serviceWSDL = attr.getNamedItem("hrefSchema").getNodeValue();       
           return serviceWSDL;
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }

    public static String getLocalWSAMIfromWCU (String wcu)     
    {
        try
        {        
            javax.xml.parsers.DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
            javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
           //org.w3c.dom.Document doc = org.apache.axis.utils.XMLUtils.newDocument(wcu);       
            org.w3c.dom.Document doc = db.parse(wcu);
            
           org.w3c.dom.Node wsdlConcrete = doc.getElementsByTagName("Local").item(0);
           org.w3c.dom.NamedNodeMap attr = wsdlConcrete.getAttributes();
           String serviceWSDL = attr.getNamedItem("hrefSchema").getNodeValue();       
           return serviceWSDL;
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }       
    }

    public static String getRemoteWSAMIfromWCU (String wcu)     
    {
        try
        {
            javax.xml.parsers.DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
            javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
           //org.w3c.dom.Document doc = org.apache.axis.utils.XMLUtils.newDocument(wcu);       
            org.w3c.dom.Document doc = db.parse(wcu);
            
           org.w3c.dom.Node wsdlConcrete = doc.getElementsByTagName("Remote").item(0);
           org.w3c.dom.NamedNodeMap attr = wsdlConcrete.getAttributes();
           String serviceWSDL = attr.getNamedItem("hrefSchema").getNodeValue();       
           return serviceWSDL;
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
       
    }

    public static String getEndPointfromWSDL (String serviceWSDL)
    {
        try
        {
            /*
            javax.xml.parsers.DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
            javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
            org.w3c.dom.Document doc = db.parse(serviceWSDL);
             */
            java.net.URL url = new java.net.URL(serviceWSDL);
            java.io.InputStream istream = url.openStream();
            org.xml.sax.InputSource isource = new org.xml.sax.InputSource(istream);
            
            org.apache.xerces.parsers.DOMParser domParser = 
                    new org.apache.xerces.parsers.DOMParser();
            domParser.parse(isource);
            org.w3c.dom.Document doc = domParser.getDocument();


           org.w3c.dom.Node endPoint = doc.getElementsByTagNameNS("*","address").item(0);

           org.w3c.dom.NamedNodeMap attr = endPoint.getAttributes();
           String serviceAddr = attr.getNamedItem("location").getNodeValue();       
           return serviceAddr;
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
       
    }

    public void setWSAMIBinDir(String _wsamiBinDir) {
        wsamiBinDir = _wsamiBinDir;
    }

    public String getWSAMIBinDir() {
        return wsamiBinDir;
    }

    public void setURAddr(String _URAddr) {
        URAddr = _URAddr;
    }

    public String getURAddr() {
        return URAddr;
    }
    
    public String[] getLocalServicesEP(String WAU) {
        Vector eps = new Vector();
        DeployedService[] ds = getDeployedServices();
	String[] hostAddresses = (new IFUtils()).getLocalHostAddresses();

        for (int i=0; i<ds.length; i++) {
            WSAMIServiceDocument WSD = ds[i].getWSAMIServiceDocument();
            if (WSD.getAbstract().equals(WAU)){

		String addr = ds[i].getEndPoint();
		int indexLocalHost = addr.indexOf("localhost");
		if (indexLocalHost != -1 && hostAddresses !=null && hostAddresses.length > 0) {
		    for (int j=0; j<hostAddresses.length; j++) {
			String beforeLocalHost = addr.substring(0,indexLocalHost);
			String afterLocalHost = addr.substring(indexLocalHost+"localhost".length());
			addr = beforeLocalHost + hostAddresses[j] + afterLocalHost;
			eps.add(addr);
		    } 
                } else
		    eps.add(ds[i].getEndPoint());
	    }
        }

        String[] epsArray = new String[eps.size()];
        eps.copyInto(epsArray);
        return epsArray;
    }
    
}
