package CoreBroker.CSoap;

//import org.apache.axis.wsdl.WSDL2Java;
import CoreBroker.BrokerWrapper;
import CoreBroker.DeployedService;
import CoreBroker.CBException;

import WSAMI.*;
import java.util.*; 

import org.w3c.dom.*;
import javax.xml.parsers.*; 

import CoreBroker.Utils;
import CoreBroker.Syslog;

public class CSoapBrokerWrapper extends CoreBroker.BrokerWrapper {
   
    private CSoapBrokerWrapper(String side) {
        super(); 
        if (side.equals(BROKERWRAPPER_SERVERSIDE)) {

            ClassLoader c = Thread.currentThread().getContextClassLoader();
            java.net.URL u = c.getResource("csoap-config.wsdd");
            org.apache.xerces.parsers.DOMParser parserDOM = new org.apache.xerces.parsers.DOMParser();
            try {
                parserDOM.parse(u.toString());
            } catch (org.xml.sax.SAXException ex) {
               //throw new CBException("broker01","error reading server-config.wsdd");
                /*
                 *
                 *  initialize with the standard default wsdd 
                 *
                 */
            } catch (java.io.IOException ex) {
               //throw new CBException("broker01","server-config.wsdd not found");            
                /*
                 *
                 *  initialize with the standard default wsdd 
                 *
                 */            
            }
            setServerConfig((Document)parserDOM.getDocument());                    
             
            java.io.InputStream is = c.getResourceAsStream("csoap-config.wsdd");
            csoap.GlobalManaging.receiveWsdd(is);
            
        } else {
            ClassLoader c = Thread.currentThread().getContextClassLoader();
            java.io.InputStream is = c.getResourceAsStream("csoap-config.wsdd");
            csoap.GlobalManagingClient.receiveWsdd(is);

        }
    }
   
    public static BrokerWrapper makeServerInstance() {
        if (_selfServerSide == null) 
            _selfServerSide = new CSoapBrokerWrapper(BROKERWRAPPER_SERVERSIDE);          
        return _selfServerSide;
    }   

    public static BrokerWrapper makeClientInstance() {
        if (_selfClientSide == null) 
            _selfClientSide = new CSoapBrokerWrapper(BROKERWRAPPER_CLIENTSIDE);          
        return _selfClientSide;
    }   

   
    public void deployService(Node service, String path, String context) 
            throws CBException {

        org.apache.xerces.parsers.DOMParser parserDOM = new org.apache.xerces.parsers.DOMParser();                
        Document specificBrokerServiceInfo = null;
        
        /*
         *  retrieve service name from wsami deploy xml
         */
        String serviceName = null;
        try{
            serviceName = service.getAttributes().getNamedItem("name").getNodeValue();
        } catch (Exception ex) {
            throw new CBException("deployverify01","Invalid service name ");            
        } 
        
        /*
         *  retrieve instance URL of the service instance
         *  from XML deploy file
         */
        String instanceURL = getTagValue(service, "instance_URL");        
        
        /*
         *  retrieve WSAMI abstract, service and customizer document names
         *  from XML deploy file
         */
        String WSAMIAbstractFileName = getTagValue(service, "wad");
        if (WSAMIAbstractFileName != null)
            WSAMIAbstractFileName = path.concat(WSAMIAbstractFileName);
        /*
        else
            throw new CBException("deployverify00","WSAMI Abstract document not defined in XML deployment file");
         */

        String WSAMIServiceFileName = getTagValue(service, "wsd"); 
        if (WSAMIServiceFileName != null)
            WSAMIServiceFileName = path.concat(WSAMIServiceFileName);
        /*
        else
            throw new CBException("deployverify00","WSAMI Service document not defined in XML deployment file");
         */
       
        String WSAMICustomizerFileName = getTagValue(service, "wcd");
        if (WSAMICustomizerFileName != null)
            WSAMICustomizerFileName = path.concat(WSAMICustomizerFileName);       
       
 
        /*
         *  retrieve WSDL concrete file name from XML deploy file
         */  
        String WSDLFileName = null;
        try {
            WSAMIServiceDocument wsd = new WSAMIServiceDocument(path, WSAMIServiceFileName);                       
            java.net.URL urlWSDLConcrete = new java.net.URL(wsd.getConcrete());
            
            if (CoreBroker.IFUtils.isLocalAddress(urlWSDLConcrete.getHost()))
                WSDLFileName = CoreBroker.IFUtils.localURI2localfile(urlWSDLConcrete,path,context);
                     
            /*
             * if the WSDL concrete document is on local machine : 
             *     trasform the http address retrieved in WSAMI Service Document
             *      into absolute path on local filesystem 
             * else keep the http address
             */
            
        } catch(java.net.MalformedURLException ex) {
            throw new CBException("deployverify01","Invalid address "+WSDLFileName+" for WSDL concrete.");            
        }
       
     
       /*
        *   retrieve from XML deploy file if
        *   the service is a local/remote 
        *   customizer or if it is not:
        *       - null = is not a customizer
        *       - "local" or "remote" = is a local / remote customizer
        */ 

       String customizerType = null; 
       NamedNodeMap svcAttrs = service.getAttributes();
       if ((svcAttrs.getNamedItem("customizer") != null)) 
            customizerType = svcAttrs.getNamedItem("customizer").getNodeValue();

        /*
        *   retrieve from XML deploy file 
        *   the deployment scope for the service:
        *   Application (default), Session, Request
        */
       String scope = getTagValue(service, "scope");
       if (scope == null)
           scope = "Application";
       
       /*
        *   retrieve implementation type
        *   (currently only java class is supported
        *   EJB is not supported)
        */
       String typeImpl = null;
       String className = null;
       String packageName = null;
       /*
        *   Useful for EJB support:
        *
        *   String jndiName = null;
        *   String homeClass = null;
        */
       Node implNode = null;
       NodeList svcChildren = service.getChildNodes();
       for (int i=0; i<svcChildren.getLength(); i++) {
            Node n = svcChildren.item(i);
            if (n.getNodeName() == "implementation") {                    
                implNode = n;
                break;
            }
       }           
       if (implNode != null){
           typeImpl = getTagValue(implNode, "type");

           if (typeImpl.equals("javaclass")) {
               className = getTagValue(implNode, "class");                 
                packageName = getTagValue(implNode, "package");                      
           } 
       }

       
       /*
        *   retrieve endpoint and serviceName
        *   from WSDL concrete file
        */    
        String serviceNameWSDL = null;
        String endPoint = null;
        try {
           parserDOM.parse(WSDLFileName);                      
           Document wsdlDoc = parserDOM.getDocument();           
           
           // retrieve ServiceNameWSDL
           NodeList list = wsdlDoc.getElementsByTagNameNS("*","port");
           NamedNodeMap attr = list.item(0).getAttributes();
           serviceNameWSDL = attr.getNamedItem("name").getNodeValue();
           
           // retrieve endpoint
           list = wsdlDoc.getElementsByTagNameNS("*","address");
           attr = list.item(0).getAttributes();
           endPoint = attr.getNamedItem("location").getNodeValue();
           
        } catch (org.xml.sax.SAXException ex) {
            throw new CBException("deployverify01","Invalid address "+WSDLFileName+" for WSDL concrete.");            
        } catch (java.io.IOException ex) {
            throw new CBException("deployverify01","Invalid address "+WSDLFileName+" for WSDL concrete.");            
        }
           
        /*
         *  generate wsdd from wsdl and use wsdd to deploy on csoap
         */
        if (scope != null && packageName != null && WSDLFileName != null) {
           
            try {
                csoap.wsdl.DeploymentGenerator wsdl2wsdd = new csoap.wsdl.DeploymentGenerator();
                wsdl2wsdd.generateWSDD(WSDLFileName,"s",false,path+"/tmp/",className, scope);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
                
            //org.apache.xerces.parsers.DOMParser parserDOM = new org.apache.xerces.parsers.DOMParser();                
            try {
                parserDOM.parse(path+"/tmp/"+packageName+"/deploy.wsdd"); 
                //parserDOM.parse(path+"/tmp/deploy.wsdd"); 
                specificBrokerServiceInfo = parserDOM.getDocument();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            

            if (customizerType != null)
               Modify_RPCProvider(specificBrokerServiceInfo, customizerType);

            // deploy on CSoap
            try {

               java.io.FileOutputStream outstr =
                     new java.io.FileOutputStream(path+"/tmp/"+packageName+"/deploy.wsdd");
               Utils.printDocToStream(specificBrokerServiceInfo, outstr);
               outstr.close();

                java.io.FileInputStream is =  
                    new java.io.FileInputStream(path+"/tmp/"+packageName+"/deploy.wsdd");
                csoap.GlobalManaging.receiveWsdd(is);
            } catch (java.io.FileNotFoundException ex) {
                throw new CBException("deploy10","Error in deployment operation");
            }catch (java.io.IOException ex) {
                throw new CBException("deploy10","Error in deployment operation");
            }
            
            
            // finally deploy the service on the Core Broker
            deploy(serviceName, endPoint, path, WSAMIAbstractFileName, WSAMIServiceFileName ,WSAMICustomizerFileName, specificBrokerServiceInfo, instanceURL );              
            
        } else
            throw new CBException("deploy10","Error in deployment operation");
   }

   private void Modify_className_and_scope(Document specificBrokerServiceInfo, String className, String scope) {   
       
        NodeList paramList = specificBrokerServiceInfo.getElementsByTagName("parameter");
        for (int i=0; i<paramList.getLength(); i++) {               
            NamedNodeMap attrs = paramList.item(i).getAttributes();
            
            if ((attrs.getNamedItem("name").getNodeValue()).equals("className")) {
                if (className != null && className != "")
                    attrs.getNamedItem("value").setNodeValue(className);
            
            } else if ((attrs.getNamedItem("name").getNodeValue()).equals("scope"))
                attrs.getNamedItem("value").setNodeValue(scope);
        }                   
   }

   
   private void Modify_RPCProvider(Document deploymentDoc, String customizerType) {

       Node svcDef = deploymentDoc.getElementsByTagName("service").item(0);
       NamedNodeMap attr = svcDef.getAttributes();
       
       // modify service definition in deploy.wsdd:
       //   from:   <service name="XYZ" provider="java:RPC" ...
       //   to:     <service name="XYZ" provider="Handler" ...
       if ((attr.getNamedItem("provider").getNodeValue()).equals("java:RPC")
	   || (attr.getNamedItem("provider").getNodeValue()).equals("JAVA:RPC")) {
           attr.getNamedItem("provider").setNodeValue("CoreBroker.CSoap.CustomizerProvider");


       }
       
       /*
       //   the handler definition is then added to the service definition node:
       //   <parameter name="handlerClass" value="CoreBroker.Axis.CustomizerProvider" />
       Element handlerChild = deploymentDoc.createElementNS("http://xml.apache.org/axis/wsdd/","parameter");
       handlerChild.setAttribute("name","handlerClass");
       handlerChild.setAttribute("value","CoreBroker.Axis.CustomizerProvider");
       svcDef.appendChild(handlerChild);
        */
   }
    
 
    protected void refreshDeploymentConfiguration() {
/*        
        // obtain the server configuration (it's the XML configuration from server-config.wsdd
        deploymentConfiguration = this.getServerConfig();
        
        //  obtain the list of services definition (the information
        //  we are interested in is the XML definition of the service 
        //  used to deploy services on Axis
        DeployedService[] deployedList = getDeployedServices();      
        for (int i=0; i<deployedList.length; i++) {
            Document doc = deployedList[i].getServiceInfo();
            if (doc!=null) {
                Node service = doc.getElementsByTagName("deployment").item(0);
                Node serviceDef = deploymentConfiguration.importNode(service, true);
                if (serviceDef != null)
                    deploymentConfiguration.getDocumentElement().appendChild(serviceDef);
            }
        }
 */
    } 
    
    
    public NDService.NDPortType getNDStub(String ndAddress) 
            throws CBException {
        try {
            java.net.URL ndAddressURL = new java.net.URL(ndAddress);
            NDService.NDPortType nd = new NDService.NDBindingStub(ndAddressURL,null);
            return nd;
        } catch (java.net.MalformedURLException ex) {
            throw new CBException("deploy02","The address of the ND service is not correct");
        }

    }    
}
