package CoreBroker.CSoap;

import java.io.*;
import java.net.*;
import org.mortbay.util.*;
import org.mortbay.http.*;
import org.mortbay.jetty.servlet.*;
import org.mortbay.http.handler.*;
import org.mortbay.servlet.*;  

import CoreBroker.Syslog;
      
public class MinimalJettyServer {
    
    public static final int TOSTOP = 0;
    public static final int RUNNING = 1;
    
    public static void main (String[] args)
            throws Exception {      
                
        /*
        * jetty.home: used by WSAMI to retrieve shared lib 
        * wsamibin.dir: used by WSAMI ND for event messaging with SLPMGR.sh
        * UR: used by WSAMI ND to call UR repository
        *
        */
        String home = System.getProperty("jetty.home",".");
        String wsamiBinDir = System.getProperty("wsamibin.dir","/opt/wsami/dist/pda/bin");
        //String URAddr = System.getProperty("UR","http://128.93.8.102:8080/services/UR");
	String URAddr = System.getProperty("UR","");
                 
        if (args.length >0 && args[0].equals("stop")) {
            stop(wsamiBinDir);
            return;
        }

        // Create the server
        HttpServer server=new HttpServer();

        // Create a port listener
        SocketListener listener=new SocketListener();
        listener.setPort(8080);
        server.addListener(listener);

        /*
        *      create root/servlet context
        */    
        HttpContext contextROOT = new HttpContext();
        contextROOT.setContextPath("/root");

        // Serve static content from the context
        contextROOT.setResourceBase(home+"/root/");
        contextROOT.addHandler(new ResourceHandler());
        server.addContext(contextROOT);

        // Create a servlet container in root context  
        contextROOT.setClassPath(home+"/root/servlet/");
        ServletHandler servletsROOT = new ServletHandler();
        contextROOT.addHandler(servletsROOT);

        // Map a servlet onto the container
        servletsROOT.addServlet("Invoker","/servlet/*",
        "org.mortbay.jetty.servlet.Invoker");

        /* end root/servlet context */


        /*
        *      Create wsami context 
        */
        HttpContext context = new HttpContext();
        context.setContextPath("/wsami");

        // Serve static content from the context
        context.setResourceBase(home+"/wsami/");
        context.addHandler(new ResourceHandler());
        server.addContext(context);

        // Create a servlet container
        ServletHandler servlets = new ServletHandler();
        context.addHandler(servlets);

        servlets.addServlet("Invoker","/servlet/*",
               "org.mortbay.jetty.servlet.Invoker");

        /*
        *	Version supporting wsami and Axis
        *
        servlets.addServlet ("AdminServlet", "/AdminServlet", "org.apache.axis.transport.http.AdminServlet");
        servlets.addServlet ("AxisServlet",  "/AxisServlet", "org.apache.axis.transport.http.AxisServlet");
        servlets.addServlet ("WSAMIServlet",  "/WSAMIServlet", "CoreBroker.Axis.WSAMIServlet");
        servlets.addServlet ("WSAMIServlet",  "/services/*", "CoreBroker.Axis.WSAMIServlet");
        */

        /*
        *	Version supporting CSoap
        */
        ServletHolder adminServlet = servlets.addServlet ("AdminServlet", "/AdminServlet", "CoreBroker.AdminServlet");
        adminServlet.setInitParameter("factoryname","CoreBroker.CSoap.CSoapBrokerWrapper");
        adminServlet.setInitParameter("config","wsami.xml");
        adminServlet.setInitParameter("context","wsami");
        adminServlet.setInitParameter("port","8080");
        adminServlet.setInitParameter("services","services");
        adminServlet.setInitParameter("wsamiBinDir",wsamiBinDir);
        if (URAddr != null && !URAddr.equals("")) adminServlet.setInitParameter("UR",URAddr);
        
        /* end wsami context */

        /*
        *  Create Services context 
        */
        HttpContext contextServices = new HttpContext();
        contextServices.setContextPath("/services");

        // Serve static content from the context
        contextServices.setResourceBase(home+"/services/");
        contextServices.setClassPath(home+"/services/WEB-INF/classes");
        contextServices.setClassPath(home+"/services/WEB-INF/lib");
        contextServices.addHandler(new ResourceHandler());
        server.addContext(contextServices);

        // Create a servlet container
        ServletHandler servletsServices = new ServletHandler();
        contextServices.addHandler(servletsServices);
        ServletHolder wsamiServlet = servletsServices.addServlet ("WSAMIServlet",  "/WSAMIServlet", "CoreBroker.CSoap.WSAMIServlet");
        servletsServices.addServlet ("CSoapServlet", "/*", "CoreBroker.CSoap.WSAMIServlet");

        /* end Service context */



        /*
        *  Start the http server
        */
        server.start ();
        adminServlet.setInitOrder(1);
        adminServlet.start();

        //Syslog.debug(this,free: " + Runtime.getRuntime().freeMemory());
        //Syslog.debug(this,"total: " + Runtime.getRuntime().totalMemory());

        /*
        *      File .jetty_running is used to verify
        *      if the user has required to stop the
        *      http server and allow a clean stop
        *      and call destroy on WSAMIServlet/CSoapServlet
        */
        FileOutputStream fos = new FileOutputStream(wsamiBinDir+"/.jetty_running");
        int running = RUNNING+48; //48 is the char of '0'
        fos.write(running);
        fos.close();

        while (true) {
            try {                        
                Thread.sleep(20000);
                FileInputStream fis = new FileInputStream(wsamiBinDir+"/.jetty_running");            
                running = fis.read(); 
                fis.close();
                Syslog.debug(MinimalJettyServer.class,"Check stop: "+running);

                if (running !=1 && running == TOSTOP+48) { //48 is the char of '0'        
                    server.stop();
                    return;
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }       
        }
    }

    public static void stop(String wsamiBinDir) {
        try {
            FileOutputStream fos = new FileOutputStream(wsamiBinDir+"/.jetty_running");
            int running = TOSTOP+48; //48 is the char of '0'
            fos.write(running);
            fos.close();  
        } catch (Exception ex) {
            ex.printStackTrace();
        }   
    }

}
