package CoreBroker;

import java.net.HttpURLConnection;
import java.net.URL;

import org.w3c.dom.*;
import javax.xml.parsers.*;
import org.xml.sax.*;


/////////////////////////////////
/*
 * CLBaseClass -> CmdLineBaseClass
 *
 */
///////////////////////////////////
public class Deployer extends CLBaseClass {
    
    String addrCoreBroker;
    String configFileName;
    String[] fileNamesDescr;
    String[] fileNamesImpl;    
    int nRetry = 1;

    public void call() {       
            
        /*
         *  create a connection to the Core Broker
         */
        HttpURLConnection urlConn = null;
        try { 
            URL url = new URL(addrCoreBroker);
            urlConn = (HttpURLConnection)url.openConnection(); 
        } catch (java.net.MalformedURLException ex) {
            Syslog.error(this,"Error connecting to the Core Broker");
        } catch (java.io.IOException ex) {
            Syslog.error(this,"Error connecting to the Core Broker");            
        }
        urlConn.setDoInput (true);
        urlConn.setDoOutput (true);
        urlConn.setUseCaches (false);
        urlConn.setRequestProperty("User-Agent", "Profile/MIDP-1.0 Configuration/CLDC-1.0");            
        urlConn.setRequestProperty("Content-Language", "en-US");
        urlConn.setRequestProperty("content-type", "application/x-www-form-urlencoded");

        /*
         *  try nRetry times to connect to 
         *  the Core Broker, default is 
         *  to try only once and exit
         *  if no connection is possible
         */
        java.io.OutputStream os = null;                        
        for (int i=0; i<nRetry; i++) {
            Syslog.info(this,"trying to connect to WSAMI Core Broker at address "+addrCoreBroker+"...");
            try {
                os = urlConn.getOutputStream();
            } catch (java.io.IOException ex) {
                os = null;
            }
            if (os != null)
                break;
        }
        if (os == null) {
            Syslog.error(this,"Connection failed");
            return;
        } else
            Syslog.info(this,"Connection established. sending file...");

        /*
         *  read deployment file and insert 
         *  the installation content that 
         *  is dynamically generated 
         */       
        //String installFileName = "install_"+configFileName;
        
        
        java.io.ByteArrayOutputStream baos = new java.io.ByteArrayOutputStream();
        
        
        try {
            java.io.FileInputStream deployFile = new java.io.FileInputStream(configFileName);        
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();        
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document d = db.parse(deployFile);

            Element root = d.getDocumentElement();
            String deploymentType = root.getNodeName();
            if (deploymentType.equals("deploy")) {
                Node serviceElem = d.getElementsByTagName("service").item(0);
                String serviceName = serviceElem.getAttributes().getNamedItem("name").getNodeValue();
                Element installElem = d.createElement("install");
                installElem.setAttribute("name",serviceName);
                serviceElem.appendChild(installElem);

                java.util.Properties props = System.getProperties();
                String separator = props.getProperty("file.separator");
                String absolutePath;
                String fileName ;

                /*
                 *  Add description files to xml installation/deploy file
                 */
                if (fileNamesDescr != null) {            
                    for (int i=0; i<fileNamesDescr.length; i++) {                
                        java.io.File file = new java.io.File(fileNamesDescr[i]);

                        if (!file.isFile()) 
                            Syslog.error(this,"Error installing file "+fileNamesDescr[i]);
                        else {
                            String fileAbsoluteName = file.getAbsolutePath();
                            int indexSeparator = fileAbsoluteName.lastIndexOf(separator);
                            if (indexSeparator != -1) {                    
                                absolutePath = fileAbsoluteName.substring(0,indexSeparator);
                                fileName = fileAbsoluteName.substring(indexSeparator+1);

                                Element fileElem = d.createElement("file");
                                fileElem.setAttribute("type","descr");
                                fileElem.setAttribute("name",fileName);
                                fileElem.setAttribute("location",absolutePath);
                                installElem.appendChild(fileElem);
                            } else 
                                Syslog.error(this,"Error installing file "+fileNamesDescr[i]);
                        }
                    }
                }

                /*
                 *  Add implementation files to xml installation/deploy file
                 */
                if (fileNamesImpl != null) {            
                    for (int i=0; i<fileNamesImpl.length; i++) {                
                        java.io.File file = new java.io.File(fileNamesImpl[i]);

                        if (!file.isFile())
                            Syslog.error(this,"Error installing file "+fileNamesImpl[i]);
                        else {
                            String fileAbsoluteName = file.getAbsolutePath();

                            int indexSeparator = fileAbsoluteName.lastIndexOf(separator);
                            if (indexSeparator != -1) {                    
                                absolutePath = fileAbsoluteName.substring(0,indexSeparator);
                                fileName = fileAbsoluteName.substring(indexSeparator+1);

                                String typeImpl = null;
                                if (fileName.endsWith("jar"))
                                    typeImpl ="implJar";
                                else if (fileName.endsWith("class"))
                                    typeImpl ="implClass";

                                Element fileElem = d.createElement("file");
                                fileElem.setAttribute("type",typeImpl);
                                fileElem.setAttribute("name",fileName);
                                fileElem.setAttribute("location",absolutePath);
                                installElem.appendChild(fileElem);

                                //xmlContent = xmlContent.concat("<file type=\""+typeImpl+"\" name=\""+fileName+"\" location=\""+absolutePath+"\" />\n");

                            } else 
                                Syslog.error(this,"Error installing file "+fileNamesImpl[i]);
                        }

                    }
                }
            } // if (deploymentType.equals("deploy"))
           
            

            //java.io.FileOutputStream fos = new java.io.FileOutputStream(installFileName);
            //CoreBroker.Utils.printDocToStream(d, fos);
            CoreBroker.Utils.printDocToStream(d, baos);
            
        } catch (java.io.FileNotFoundException ex) {
            Syslog.error(this,configFileName+"not found");
        } catch (java.io.IOException e) {
            Syslog.error(this,"Error reading file "+configFileName);
        } catch (org.xml.sax.SAXException ex) {
            Syslog.error(this,"Error creating install/deploy file ");
        } catch (javax.xml.parsers.ParserConfigurationException ex) {
            Syslog.error(this,"Error creating install/deploy file ");
        }        
        
        /*
         *  read configuration file and try
         *  to send its content to the
         *  core broker
         */
        byte[] data = null;
//        try {
            //java.io.FileInputStream fIn = new java.io.FileInputStream(installFileName);        
            //data = new byte[fIn.available()];
            //fIn.read(data);
            
            data = baos.toByteArray();
     
        try {
            os.write(data);            
            os.close();
            java.io.InputStreamReader isr = new java.io.InputStreamReader(urlConn.getInputStream());          
            isr.close();                
            Syslog.info(this,"File succesfully sent.");
        } catch (java.io.IOException e) {
            Syslog.error(this,"Error sending file "+configFileName+" to Core Broker");
            
        } 
        

    }


    protected void addOptions() {
        options.put("deploy",new CLOptionDescr("deploy",1,"The name of the (un)deployment file"));
        options.put("CoreBrokerAddr",new CLOptionDescr("CoreBrokerAddr","The address of the Core Broker on the local machine","http://localhost:8080/wsami/AdminServlet",null));
        options.put("descr",new CLOptionDescr("descr",ATLEASTONE,"The name of the files used to describe the service (wsami and wsdl files)"));
        options.put("impl",new CLOptionDescr("impl",ATLEASTONE,"The name of the files that implement the service (class files)"));       
        
        //options.put("retry",new CLOptionDescr("retry","The number of times AdminCBClient tries to send a request to deploy to the server", "1", null));
    }
   
    protected int setNParams() {
        //return 1;
        return 0;
    }
   
    protected String setParamsDescription() {
        //return "deployment_filename";
        return null;
    }
   
    protected void setValues() {
        addrCoreBroker = "http://localhost:8080/wsami/AdminServlet";
        if (getStringOption("CoreBrokerAddr") != null)
            addrCoreBroker = getStringOption("CoreBrokerAddr");
        /*
        nRetry = 1;
        if (getStringOption("retry") != null)
            nRetry = new Integer(getStringOption("retry")).intValue();
            */
        
        fileNamesDescr = getArrayStringOption("descr");
        fileNamesImpl = getArrayStringOption("impl");        
        
        //configFileName = params[0];
        configFileName = getStringOption("deploy");
    }

   
    public static void main(String arg[]) {
        Deployer client = new Deployer();
        if (client.run(arg))
            client.call();     
    }

}

