package CoreBroker;

import java.net.*;
import java.io.OutputStream;

import org.w3c.dom.*;
import java.util.*;

import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.JarInputStream;

import java.io.IOException;
import java.io.InputStream;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;




public class IFUtils {

    
private static int countGetHost = 0;    
    
    private native String[] getIFAddrs();
    private native void iwconfig2();
    private native int[] getACInfo();
    private native NetworkIFDescr[] getNetworkIFInfo();
    
    private native int iproute(String gwIP);

    public static int  VERSION_MAJOR = 0;
    public static int  VERSION_MINOR = 1;
    public static int  FLAGS = 2;
    public static int  ACLINE_STATUS = 3;
    public static int  BATTERY_STATUS = 4;
    public static int  BATTERY_FLAGS = 5;
    public static int  BATTERY_PERC = 6;
    public static int  BATTERY_TIME = 7;
    public static int  USING_MINUTES = 8;
    
    public static int  ADHOC = 1;
    public static int  INFRABASED = 2;
    public static int  INFRABASED_WIRED = 3;
    
    private static boolean JavaNetNetworkInterfaceClass_Supported = true;
    
    static  {
        java.util.Properties props = System.getProperties();
        String osname = props.getProperty("os.name");

        if (osname.indexOf("Linux") != -1) {
            
            String sharedLibPath = null;
            if (System.getProperty("catalina.home") != null) 
                sharedLibPath = System.getProperty("catalina.home")+"/shared/lib";
            else if (System.getProperty("jetty.home") != null) 
                sharedLibPath = System.getProperty("jetty.home")+"/lib";

            String libIW = sharedLibPath+"/libiw.so.25"; 
            System.load(libIW);                

            String libIW2 = sharedLibPath+"/libiw.so"; 
            System.load(libIW2);                

            String libIFMgr = sharedLibPath+"/libifacemgr.so"; 
            System.load(libIFMgr);
        } else if (osname.indexOf("Windows") != -1) {
            
            String sharedLibPath = null;
            if (System.getProperty("catalina.home") != null) 
                sharedLibPath = System.getProperty("catalina.home")+"\\shared\\lib";
            else if (System.getProperty("jetty.home") != null) 
                sharedLibPath = System.getProperty("jetty.home")+"\\lib\\classpath";

            String libIFMgr = sharedLibPath+"\\ifacemgr.dll"; 
         
            System.load(libIFMgr);
            
        }                

    }    
    
    
    public String[] getIFAddresses() {
        Vector addrVector = new Vector();
        String[] returnedValue = getIFAddrs();
        
        for (int i=0; i<returnedValue.length; i++)
            if (returnedValue[i] != null && !returnedValue[i].equals("127.0.0.1"))
                addrVector.add(returnedValue[i]);
        
        String[] addrArray = new String[addrVector.size()];
        addrVector.copyInto(addrArray);
        return addrArray;	
        
    }
    
    
    public String[] getLocalHostAddresses() {            
        Vector addrVector = new Vector();
                                
        try {
            if (JavaNetNetworkInterfaceClass_Supported) {
                for( Enumeration ifaces = NetworkInterface.getNetworkInterfaces(); ifaces!=null && ifaces.hasMoreElements(); ) {
                    NetworkInterface ni = (NetworkInterface) ifaces.nextElement();
                    for( Enumeration ips = ni.getInetAddresses(); ips.hasMoreElements(); ) {
                        InetAddress addr = (InetAddress) ips.nextElement();
                        if (!addr.isLoopbackAddress())
                            addrVector.add(addr.getHostAddress());
                    }
                }
            } else {
                try {
                    String[] returnedValue = getIFAddrs();
                    for (int i=0; i<returnedValue.length; i++)
                        if (returnedValue[i] != null && !returnedValue[i].equals("127.0.0.1"))
                            addrVector.add(returnedValue[i]);
                } catch (Exception ex2) {
                    ex2.printStackTrace();
                }                
            }
        } catch (NoClassDefFoundError ex) {
            //ex.printStackTrace();
            Syslog.warning(this,"java.net.NetworkInteface class not supported");
            JavaNetNetworkInterfaceClass_Supported = false;
            
            try {
                String[] returnedValue = getIFAddrs();
                for (int i=0; i<returnedValue.length; i++)
                    if (returnedValue[i] != null && !returnedValue[i].equals("127.0.0.1"))
                        addrVector.add(returnedValue[i]);
            } catch (Exception ex2) {
                ex2.printStackTrace();
            }

        } catch (java.net.SocketException ex) {
            ex.printStackTrace();
            
            try {
                String hostname = java.net.InetAddress.getLocalHost().getHostAddress();
                addrVector.add(hostname);
            } catch (Exception ex2) {
                ex2.printStackTrace();
            }
 
        }
        
        
        String[] addrArray = new String[addrVector.size()];
        addrVector.copyInto(addrArray);
        return addrArray;	      
    }

    
    public static boolean isLocalAddress(String host) {   

        CoreBroker.IFUtils util = new CoreBroker.IFUtils();

        String[] localHostAddrs = util.getLocalHostAddresses();

        try {
            InetAddress[] hostAddrs = InetAddress.getAllByName(host);

            for (int i=0; i<hostAddrs.length; i++) {
		if (hostAddrs[i].getHostAddress().equals("127.0.0.1") ||
                    hostAddrs[i].getHostAddress().equals("localhost"))
                            return true;
                for (int j=0; j<localHostAddrs.length; j++) {
                    if (hostAddrs[i].getHostAddress().equals(localHostAddrs[j]) ||
                        hostAddrs[i].getHostAddress().equals("127.0.0.1") ||
                        hostAddrs[i].getHostAddress().equals("localhost")) {
                            return true;
                    }
		}
	    }
            return false;    
        } catch (java.net.UnknownHostException ex) {            
            return false;
        }   
        
    }
            
    
    public static String localURI2localfile(URL url, String path, String context) 
            throws CBException {
                
        int indexWSAMIDir = url.getFile().indexOf("/"+context+"/");
        if (indexWSAMIDir == -1)
            throw new CBException("file","Error: file is in local address but is defined outside wsami context ="+context);

        int length = url.getFile().length();
        String file = url.getFile().substring(indexWSAMIDir+7,length);
        file = path.concat(file);
        return file;
    }
    
    
    
    public static boolean isSameURL(String u1, String u2) {
        
        try {
            URL url1 = new URL(u1);
	    boolean isURL1Local = false;
            URL url2 = new URL(u2);
	    boolean isURL2Local = false;
            CoreBroker.IFUtils util = new CoreBroker.IFUtils();
            String[] u1HostAddrs;
            String[] u2HostAddrs; 
            
            String u1Host = url1.getHost();
            if (u1Host.equals("localhost") || u1Host.equals("127.0.0.1")) {
                u1HostAddrs = util.getLocalHostAddresses();                
		isURL1Local = true;
            } else {
                InetAddress[] addrs1 = InetAddress.getAllByName(u1Host);
                u1HostAddrs = new String[addrs1.length];
                for (int i=0; i<addrs1.length; i++)
                    u1HostAddrs[i] = addrs1[i].getHostAddress();
            }
                
            String u2Host = url2.getHost();
            if (u2Host.equals("localhost") || u2Host.equals("127.0.0.1")) {
                u2HostAddrs = util.getLocalHostAddresses();                
		isURL2Local = true;
            } else {
                InetAddress[] addrs2 = InetAddress.getAllByName(u2Host);
                u2HostAddrs = new String[addrs2.length];
                for (int i=0; i<addrs2.length; i++)
                    u2HostAddrs[i] = addrs2[i].getHostAddress();
            }   

	    if (isURL1Local && isURL2Local) 
		return true;
            for (int i=0; i<u1HostAddrs.length; i++)
                for (int j=0; j<u2HostAddrs.length; j++)
                    if (u1HostAddrs[i].equals(u2HostAddrs[j]) &&
                        url1.getFile().equals(url2.getFile())) {
                            return true;
                    }
            return false;

        } catch (java.net.MalformedURLException ex) {
            ex.printStackTrace();
            return false;
        } catch (java.net.UnknownHostException ex) {
            ex.printStackTrace();
            return false;
        }
    }    
    
   
    public static boolean isSameHost(String h1, String h2) {
        
        try {
            CoreBroker.IFUtils util = new CoreBroker.IFUtils();
            String[] h1HostAddrs;
            String[] h2HostAddrs; 
            
            if (h1.equals("localhost") || h1.equals("127.0.0.1"))
                h1HostAddrs = util.getLocalHostAddresses();                
            else {
                InetAddress[] addrs1 = InetAddress.getAllByName(h1);
                h1HostAddrs = new String[addrs1.length];
                for (int i=0; i<addrs1.length; i++)
                    h1HostAddrs[i] = addrs1[i].getHostAddress();
            }
                
            if (h2.equals("localhost") || h2.equals("127.0.0.1"))
                h2HostAddrs = util.getLocalHostAddresses();                
            else {
                InetAddress[] addrs2 = InetAddress.getAllByName(h2);
                h2HostAddrs = new String[addrs2.length];
                for (int i=0; i<addrs2.length; i++)
                    h2HostAddrs[i] = addrs2[i].getHostAddress();
            }   
            
            for (int i=0; i<h1HostAddrs.length; i++)
                for (int j=0; j<h2HostAddrs.length; j++)
                    if (h1HostAddrs[i].equals(h2HostAddrs[j])) {
                            return true;
                    }
            return false;

        } catch (java.net.UnknownHostException ex) {
            ex.printStackTrace();
            return false;
        }
    }    
    
    
    public int[] getEnergyInfo() {
        int[] acInfo = getACInfo();
        return acInfo;
    }    
    
    public void getIFInfo() {
        iwconfig2();
    }
    
    public NetworkIFDescr[] getNetworkInfo() {
        NetworkIFDescr[] nwifInfo = getNetworkIFInfo();
        Vector nwInfoVector = new Vector();
        
        for (int i=0; i<nwifInfo.length; i++)
            if (nwifInfo[i] != null && !nwifInfo[i].ipAddr.equals("127.0.0.1"))
                nwInfoVector.add(nwifInfo[i]);
        
        NetworkIFDescr[] nwInfoArray = new NetworkIFDescr[nwInfoVector.size()];
        nwInfoVector.copyInto(nwInfoArray);
        return nwInfoArray;        
    }
    
    public int IProute(String gwIP) {
        // 0 : OK
        // -1 : error setting default gw
        return iproute(gwIP);
    }    
        

    public static String getFavoritesDir() {
        if (System.getProperty("catalina.home") != null)
                return System.getProperty("catalina.home");
        else if (System.getProperty("jetty.home") != null)
                return System.getProperty("jetty.home");
        else
                return null;
    }

}
