package WSAMI;

import org.w3c.dom.*;
import javax.xml.parsers.*;
import org.xml.sax.*;

import CoreBroker.Syslog;
import CoreBroker.CBException;

public class WSAMIDocument
{
    protected String name;
    protected String targetNamespace;
   
    protected Document getDocumentFromFilename(String XSDfilesPath, String filename) 
            throws CBException {
                
        org.apache.xerces.parsers.DOMParser domParser = new org.apache.xerces.parsers.DOMParser();
        try {
            domParser.setFeature("http://xml.org/sax/features/validation", true);
            domParser.setFeature("http://apache.org/xml/features/validation/schema", true);
        } catch (org.xml.sax.SAXNotRecognizedException ex) {
            throw new CBException("parsing00","XML validation not supported by parser");
        } catch (org.xml.sax.SAXNotSupportedException ex) {
            throw new CBException("parsing00","XML validation not supported by parser");            
        }
        
        org.xml.sax.EntityResolver er = new XSDEntityResolverHandler(XSDfilesPath);
        domParser.setEntityResolver(er);
        
        DeployErrorHandler errorHandler = new DeployErrorHandler();
        domParser.setErrorHandler (errorHandler);
        
        try {
            domParser.parse(new org.xml.sax.InputSource(new java.io.FileInputStream(filename)));
        } catch (org.xml.sax.SAXException ex) {
            throw new CBException("parsing00","Error parsing "+filename);
        } catch (java.io.FileNotFoundException ex) {
            throw new CBException("parsing00","File "+filename+" not found");
        } catch (java.io.IOException ex) {
            throw new CBException("parsing00","Error reading "+filename);
        }

        if (errorHandler.error)
            throw new CBException("XMLparsing00","Error validating XML data: "+errorHandler.error_descr);


        return domParser.getDocument();                    
    }

   
    public void setName(String _name)
    {
      name = _name;
    }

   public void setTargetNamespace(String _ns)
   {
     targetNamespace = _ns;
   }

   public String getName()
   {
     return name;
   }

   public String getTargetNamespace()
   {
     return targetNamespace;
   }

    public Document getXML() {
       
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();        
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document d = db.newDocument();

            Element root = d.createElement("wsami");
            root.setAttribute("xmlns","http://www-rocq.inria.fr/arles/wsami");
            root.setAttribute("xmlns:xsi","http://www.w3.org/2001/XMLSchema-instance");
            root.setAttribute("xsi:schemaLocation","http://www-rocq.inria.fr/arles/wsami WSAMI.xsd");
            d.appendChild(root);

            Element definition = d.createElement("Definition");
            definition.setAttribute("name",name);
            definition.setAttribute("targetNamespace",targetNamespace);
            root.appendChild(definition);
          
            return d;
        } catch (javax.xml.parsers.ParserConfigurationException ex) {
            return null;
        }
   }

    private class DeployErrorHandler implements ErrorHandler {
        boolean error = false;
        String error_descr = null;

        public void warning (SAXParseException e)
            throws SAXException {        
        }

        public void error (SAXParseException e)
                throws SAXException {
            error = true;
            error_descr = e.getMessage();
        }

        public void fatalError (SAXParseException e)
                throws SAXException {
            error = true;    
            error_descr = e.getMessage();
        }
    }

    private class XSDEntityResolverHandler implements EntityResolver {

        String rootPath = "";

        public XSDEntityResolverHandler(String path) {          
            rootPath = path;
        }

        public org.xml.sax.InputSource resolveEntity(java.lang.String publicId, 
                java.lang.String systemId)  {

            /*
             *  retrieve the name of the .xsd file
             *  and build the complete filename path
             *  in the local filesystem
	     *
             *  the systemId name is in urn format
             *  (ex: file:///home/filename.xsd of file:///c:/home/filename.xsd
	     *  so the separator is always "/" character
             */
  	    String separator ="/";             
            int indexSeparator = systemId.lastIndexOf(separator);
            if (indexSeparator == -1) 
                return null;

            String XSDfilename = systemId.substring(indexSeparator+1);
            String resourceName = rootPath.concat(XSDfilename);

            /*
             *  try to create InputSource from xsd filename
             */
            try {
                return new org.xml.sax.InputSource(new java.io.FileInputStream(resourceName)); 
            } catch (java.io.FileNotFoundException ex) {
                Syslog.error(this,"File "+resourceName+" not found.");
                return null;
            }
        }    

    }
}
